package byu;

/****************************************************************************
** COPYRIGHT (C):    1996 Cay S. Horstmann. All Rights Reserved.
** PROJECT:          Computing Concepts with Java
** FILE:             GraphicsApplet.java
****************************************************************************/

/**
 * A turtle graphics class for drawing simple graphics
 * @version 1.00 11 Apr 1997
 * @author Quinn Snell (BYU)
 */


import java.awt.*;
import java.awt.event.*;
import java.lang.Math;

public class Turtle
{
	protected int x = 0;				     //x-coordinate
	protected int y = 0;				     //y-coordinate
	protected boolean pendown = false;	     //pen up (true)? or down (false)?
	protected int angle = 0;			     //turtle angle, in degrees
	protected boolean speed = true;		     //is turtle slowed down (false)?
	GraphicsApplet theApplet;
	Graphics g;


	/**
	 * Constructs the turtle with initial location (xx,yy)
	 * @param gap is the graphics applet
	 */
	public Turtle(int xx, int yy, GraphicsApplet gap)
	{
		theApplet = gap;
		g = gap.graph.getGraphics();
		g.setColor(Color.black);
		GraphicsCoord coord = theApplet.getCoord();
		x = coord.xToPixelPos(xx);
		y = coord.xToPixelPos(yy);
		//GraphicsApplet.addShape(this);
	}
	/**
	 * Constructs the turtle with initial location (0,0)
	 * @param gap is the graphics applet
	 */
	public Turtle(GraphicsApplet gap)
	{	
		theApplet = gap;
		g = gap.graph.getGraphics();
		g.setColor(Color.black);
		GraphicsCoord coord = theApplet.getCoord();
		x = coord.xToPixelPos(0);
		y = coord.xToPixelPos(0);
		//GraphicsApplet.addShape(this);
	}
	/**
	 * changes the color of the lines drawn
	 * @param newcolor is the color to change to
	 */
	public void ChangeColor(Color newcolor)
	{
		g.setColor(newcolor);
	}
	//draw_it handles all the drawing
	void draw_it(int curx,int cury,int newx,int newy)
	{	

		g.drawLine(curx,cury,newx,newy);
		
		if(!speed) 
			time_delay(2000);		
			
	}
	/**
	 * forces the turtle to draw quickly.
	 * this is the default
	 */
	public void GoFast()
	{					
		speed = true;
	}
	/**
	 * forces the turtle to draw slowly
	 */
	public void GoSlow()
	{					
		speed = false;
	}
	/**
	 * Move turtle dist from current position at current angle
	 */
	public void Move(int dist)
	{																				
		int curx = x;									
		int cury = y;	
		int alpha = angle;										
		GraphicsCoord coord = theApplet.getCoord();
		x += coord.xToPixelDist(Math.cos(alpha*Math.PI/180) * dist);
		y += coord.yToPixelDist(Math.sin(alpha*Math.PI/180) * dist);
		
		if(pendown)
			draw_it(curx, cury, x, y);
	}
	/**
	 *Transport turtle to (xx, yy). Draw line if pen is down
	 */
	public void MoveTo(int xx, int yy)
	{																				
		int curx = x;									
		int cury = y;								
		GraphicsCoord coord = theApplet.getCoord();
		x = coord.xToPixelPos(xx);
		y = coord.xToPixelPos(yy);
		if(pendown)	  								
			draw_it(curx, cury, x, y);
	}
	/**
	 * turtle will draw when moved
	 */
	public boolean PenDown()
	{								
		pendown = true;
		return pendown;
	}
	/**
	 * turtle will not draw when moved
	 */
	public boolean PenUp()
	{									
		pendown = false;
		return pendown;
	}
	/**
	 * Places a string at the current location
	 * @param s is the string to be placed;
	 */
	public void PutString(String s)
	{
		g.drawString(s, x, y);
	}
	//my own time delayer for Turtle to use,uses recursion to help slow it down
	private void time_delay(int x)
	{					
		for(int i=0;i<=1000;i++)
		{							
			i = i + 2;
			i = i - 2;
			i = i + 1;
			i = i - 1;
		}
		if(x!=0)
			time_delay(x-1);
	}
	/**
	 * Rotates turtle alpha degrees, counter-clockwise
	 */
	public int Turn(int alpha)
	{							
	    //-alpha is necessary for positive counter-clockwise rotation
		int rotate = -alpha;
		while (rotate > 360) 
			rotate = rotate-360;   
		while (rotate < (-360)) 
			rotate = rotate+360;
		angle = angle+rotate;
		return angle;
	}
	/**
	 * Rotates turtle to specified angle
	 * alpha equal to zero points turtle to the right
	 * positive alpha values are counter-clockwise from 0
	 */
	public int TurnTo(int alpha)
	{						
		//"-alpha" is necessary for positive counter-clockwise rotation
		int ang = -alpha;
		while(ang > 360) 
			ang = ang-360;
		if (ang == 360) 
			ang = 0;
		angle = ang;
		return angle;
	}
}
