/*	
	Name: TCS

	Programmers:	Mark Clement	[MC]
					Jacob Derington	[JD]
					Steven Woolley  [SW]
					David Posada	[DP]
					
	Mark, Jake and Steve are at the Department of Computer Science at Brigham Young University, Provo, UT 84601
	David is now at the Department of Biochemistry, Genetics and Imunology at the University of Vigo, 36200 Vigo, Spain


	Purpose: TCS estimates a phylogenetic network from DNA sequences following the statistical parsimony algorithm
	as described in
	
	Templeton, A. R., K. A. Crandall and C. F. Sing 1992. A cladistic analysis of phenotypic 
		associations with haplotypes inferred from restriction endonuclease mapping and DNA 
		sequence data. III. Cladogram estimation. Genetics 132: 619-633.


	Contact:		dposada@uvigo.es			



	VERSION HISTORY
	--------------------------------------------------------------------------------------------------

	Version 1.04 (Jul 00)		:	Bug when collapsing sequences to haplotypes fixed [DP]
									Bug	when calculating outgroup weights for several networks [DP]
					
	Version 1.05 (Jul 00)		:	Bug that violated minimum connections fixed [MC]
									Cosmetic changes [DP]

	Version 1.06 (Oct 00)		:	Run information and outgroup weights for each haplotype (see below) 
										is send to the LOGFILE instead of to a window [DP]
									A progress bar is included that displays the progress of the program.  
										It is not linear, but will be a good indication that the program is making 
										its calculations [DP]
									TCS will display in the window the maximum number of steps (substiutions) 
										to connect parsimoniously two haplotypes with a 95% confidence [DP]
									Once in the graph,  if you double click on a haplotype, you will be able 
										to see which sequences belong to that haplotype and the ougroupt weight for 
										that haplotype (see Castelloe and Templeton 1994, MPE 3 (2): 102-113) [DP]

 	Version 1.07 (Nov 00)		:	A histogram class was added. It is there but I do not want to use it yet [DP]
	
	Version 1.08 (Feb 01)		:	Fixed bug that was creating several unconnected haplotyped (when they should be
										connected) for some big data sets [MC]
									The progress of the calculations are showed in the GUI [DP]
									Added some new classes for nesting and mutation mapping that we are not 
										releasing yet [JD]
								
	Version 1.09 (Feb 22)  		:   Removed the intermediate default labeling in display [DP]
								 	New display features [DP]

	Version 1.10 (Feb 24)		:	Fixed a bug by which lowecase or uppercase nucleotides were treated as different
										(i.e. A != a, when it should be A == a) [DP]
									Logfile output format improved [DP]
									Version information is only dependent on dna.java (before it was on VGJ.java and
										GraphWindow.java) [DP]

	Version 1.11 (12 March 01)	:	Set a switch for all output, switch for intermediate, modified print matrix	[MC]
									Evaluatemetric too small distances really bad, connectcluster added candidatedest vector 
										to hold new added intermediates on connections connectTaxa same as connectclusters [MC]
									Bestmetric passes candidates through, connectcomponents adds intermediates to candidate vector [MC]
  
 	Version 1.12 (16 March 01)  :	Made negative paths very costly. Less care about number of intermediates [MC]
	
	Version 1.13 (4 April 01)	:	Fixed again the unconnectedness bug [MC]
									A list of haplotypes is printed to the logfile [DP]
									Close file pointer after reading sequences or after errors! [DP]
									Display messages in case haplotypes differ only by missing data [DP]
									
	Version 1.14 (5 June 01)	: 	Fixed again the too close bug [JD]
									Fixed the output format for .pict (works on Macs and PC) [JD]
									Improved the Nesting algorithm, with two separate output formats into a Nesting file [JD]
									Fixed some minor bugs through out the dna.java file [JD]
									Added additional explaination through out the dna.java file [JD]
 
 
    Version 1.15 (27 February 03):	Allowed nesting on re-opened graphs and modified graphs [SW]
    								Also allowed the user to choose the connection limit [SW]
    									(both by % or actual connections) [SW]
    								Changed many static values to non-static, so it can be instantiated 
    									accurately multiple times [SW]
  									
  									
    								- Version 1.15 Additions (June 2003):
   									Added option for selecting the root (assumes that the root is a rectangular node) [SW]
    								Automatically selects down when new network is created using this select (changed from selectAll) [SW]
    								Disabled Nesting from menu [SW]
     								Added buttons to select whether to calculate connection limit or to input directly  [SW]
    								Disabled run button until a valid file is opened... Should now be able to open and 
    									run multiple files from one window sequentially [SW]
   									Moved file reading and Debug output code to a new classes [SW]
   									Allows command line operation for multiple files: [SW]
    									usage:     VGJ -toPict directory
   										Converts any .graph file in the given Directory to Pict file (same filename + .PICT) VGJ directory
    									Constructs a network for each .phylip file in the given directory
   								 	Added private boolean directAndPrintEdges(TextOutputStream out, TaxaItem root)
    									which changes the Graph to a Directed graph [SW]
     
   									- Version 1.15 Additions (July 2003):
   									Add mode -saveStand which will look at all .graph files in the given directory [SW] 
   										and save them into a standard graph format that consists of the following:
   										First line = total#nodes total#NodesRepresentingRealSequences
   										Next       = x and y coordinates for all nodes
    									Finally    = list of all connections by index
   										Note:  These indices don't corrospond directly to the taxanumber (or names obviously)
    									That information is in the logfile under the header "Haplotype list:"
    									Note:  This will only output the first graph, if there are multiple (due to unconnected parts)
    									Note:  This file is saved as the original filename + .net at the end, and is saved automatically
    									when run with just a directory as the first parameter.
    								Added option for inputing the connection limit [SW]
   										Usage VGJ directory maxconnectionlimit
    								
    								- Version 1.15 additions (January 2004)
									I commented out several of the debug log statements in the nested inner loops of the 
										main network construction algorithm, since they were using the majority of the CPU 
										on complex network reconstructions [SW]
									I fixed a bug where the "ambiguous data" variable was not being reset between individual 
										sequence comparisons, resulting in saying that sequences differed only in ambiguous 
										or gap characters, 	when really they didn't differ at all  [SW]


 	
 	Version 1.16 (16 March 04):		Put together new package [DP]. Steve reorganized source files: divided dna.java in several source files, 
  										removed some BYU files (and other sent to CLAD) and eliminated several from the EDU folder.
  									Added expiration method (but disabled) [DP]
 									The program can read IUPAC symbols and will treat them as missing data (i.e, like a ?) [DP]
							  		IUPAC codes -> 	M: A/C		S: G/C		H: A/C/T	B: C/G/T
							  						R: A/G		K: G/T		V: A/C/G	X: A/C/G/T = ?
							  						W: A/T		Y: C/T		D: A/G/T	N: A/C/G/T = ?
										Note: we could do better than this (e.g., clearly W != S) but not sure whether it is worthy [DP]		
									Reads nexus file with names consisting of one character (e.g. 'A') [DP]
									Added a function (ReadHaps) to work with inferred haplotype (for example from PHASE) and include frequency
										and confidence [DP]:
									
										ID# haplotype absfreq confidence 
										1 1222111111111211111121 1100 0.987
										2 2121112111111121111111 634 0.984
										3 2121111111111211111121 536 0.983
										4 2121112112111111112121 486 0.983
										5 1222111111111211211121 404 0.985
									
									Removed Histogram.java we are not really using this [DP]	
									Removed NestDialog.java, we are not ready [DP]
 									Enabled Spring menu [DP]
 									Fixed some (but not all) deprecated methods in Java 1.4.2 [DP]
 									Added option to display the running JVM on the title 
 										(so added a second addButton method in Lpanel.java) [DP]
									Removed XYZ axes from the graph display [DP]
									Recognize current OS
 
 
 	Version 1.17 (13 May 04):		Fixed PICT and PS outpout menu [DP]
 									Remove an extra GraphWindow.java file in the srd directory [DP] 
									Make BrowserLauncher.java class to be aprt of the clad package [DP]

  */

package clad;

import EDU.auburn.VGJ.graph.GMLobject;
import EDU.auburn.VGJ.gui.GraphWindow;

// import byu.*;
import java.awt.Color;
import java.util.Date;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.JOptionPane;


/**
 *  Description of the Class
 *
 * @author     wooo
 * @created    January 21, 2004
 */
public class dna
{

	public final static String VERSION  = "v1.17";
	public final static int BIGNUM      = 1000000000;	
	public final static boolean log     = true;	//this is set to true if when parsing the infile 1st char of 1st seq name is read
	public final static int debug       = 0x8800;	// prints debug to a logfile instead
	public static String currentOS;
	
	// 1=readfile 2=evaluatemetric 4=printcluster 0x8=connectcomponents 0x10=recalcdist 0x10000=time variable
	// 0x20=buildmatrix  0x40=PrintTaxa 0x80=print metric values for each choice 0x100=recalcmindistance
	// 0x200=connecttaxa(bestMetric 0x400=connectclusters(evaluateMetric) 0x800 print metrics from each choice
	// 0x1000=print cluster connections 0x2000 minimal cluster connection printing.
	// 0x4000=write out intermediate graph files 0x8000 added connections(connenctComponents)
	// 0x40000=cluster connection debug 0x10000 timing debug

	private GraphWindow frame;
	private int maxNameLen              = 0; // the max name length, used to build dup shapes
	private boolean warnMissing         = false; // when two sequences differ only by missing data
	private int Inum                    = 0; // the number of intermediaries jake add
	private int MaxDistance             = 5; 
	private int MinMinDist              = 0;
	private int bestclust;
	private double bestime;
	private int besttaxa;
	private Vector components           = new Vector();
	public Vector alltaxa               = new Vector();
	private Vector realtaxa             = new Vector();
	private Logger Log;
	private FileReader fileReader;
	private boolean distances           = false;	// pairwise distances among populations
	private boolean gapmode             = true;
	public boolean graph_exists;
	private String[] maxWtaxa;
	private int maxdistance             = 40; // the max distance we will pay attention to
	private boolean commandLineMax      = false;
	private String missingString        = "";
	private double[] outweights;
	private int onOffinter              = 0;  // turns on and off the intermediate label display
	private long start1;
	private String[] xlabels;
 

	/**
	 *  Constructor for the dna object
	 *
	 * @param  f  Description of the Parameter
	 */
	public dna(GraphWindow f)
	{
		frame = f;
	}


	/**  Constructor for the dna object */
	public dna() { }


	/**
	 *  Description of the Method
	 *
	 * @param  distance  Description of the Parameter
	 */
	public void SetMaxDistance(int distance)
	{
		maxdistance = distance;
		commandLineMax = true;
	}


	/**  Description of the Method */
	private void PrintTaxa()
	{

		Enumeration enumc  = components.elements();
		Enumeration enumt;
		Component tempc;
		TaxaItem tempt;

		while (enumc.hasMoreElements())
		{
			tempc = (Component) enumc.nextElement();
//			Log.dprintln("tempc id: " + tempc.id, 64);
			enumt = tempc.taxa.elements();

			while (enumt.hasMoreElements())
			{
				tempt = (TaxaItem) enumt.nextElement();
//				Log.dprintln("tempt id: " + tempt.id, 64);
//				Log.dprintln("Name: " + tempt.name, 64);
//				Log.dprintln("Num Duplicates: " + tempt.numduplicates, 64);
//				Log.dprintln("Data" + tempt.characters, 64);
			}
		}
	}


	/**
	 *  Insert the method's description here. Creation date: (1/13/00 12:21:23
	 *  AM)
	 */

	// derived from a statistical calculation at startup

	/**
	 *  add him to all of the distance vectors We only have non-infinite
	 *  distances if the taxa used to have a connection to before or after, if
	 *  connected to before it is the distance to before+1 if connected to after
	 *  it is the distance to after + distance to the other end Otherwise, it is
	 *  infinite Creation date: (11/4/99 11:42:13 AM)
	 *
	 * @param  newtaxa          Description of the Parameter
	 * @param  before           Description of the Parameter
	 * @param  before_distance  Description of the Parameter
	 * @param  after            Description of the Parameter
	 * @param  after_distance   Description of the Parameter
	 */
	private void AddIntermediate(TaxaItem newtaxa, TaxaItem before, int before_distance, TaxaItem after, int after_distance)
	{

		// newtaxa is the new intermediate
		// before is the taxa before the new intermediate in the chain
		// before_distance is the distance from before to the new intermediate
		// after is the taxa on the other end of this connection
		// after_distance is the distance from the new intermediate to the taxa on the other end of the chain.
		// As we add new taxa, we need to create their distance vectors
		// We also update everyone elses distance vectors to include this node.
//		Log.dprintln("adding intermediate " + newtaxa.id + "[" + newtaxa.name + "] between " + before.id + "[" + before.name + "][dist=" + before_distance + "]" + " and " + after.id + "[" + after.name + "][dist=" + after_distance + "]", 0x40000);

		Enumeration enumall;
		TaxaItem curtaxa;
		enumall = alltaxa.elements();

		while (enumall.hasMoreElements())
		{

			Distance newdistance;
			Distance fromnewdistance;
			Distance dummy;
			curtaxa = (TaxaItem) enumall.nextElement();

			if (curtaxa == newtaxa)
			{

				// If this is the new intermediate, just add the distance object and leave
				fromnewdistance = new Distance(newtaxa.id, curtaxa.id, 0);
				newtaxa.compdist.addElement(fromnewdistance);
				dummy = new Distance(newtaxa.id, curtaxa.id, 0);
				newtaxa.metricdist.addElement(dummy);

				continue;
			}

			// Get the distance between us and the taxa before and the distance between us and the taxa
			// at the other end
			Distance beforetaxa       = (Distance) curtaxa.compdist.elementAt(before.id);
			Distance aftertaxa        = (Distance) curtaxa.compdist.elementAt(after.id);

			// Add a new Distance to all of the distance vectors
			// If there is a current connection to the before taxa that is less than the connection to the after
			if ((beforetaxa.distance != BIGNUM) && (beforetaxa.distance < aftertaxa.distance))
			{
				newdistance = new Distance(curtaxa.id, newtaxa.id, beforetaxa.distance + before_distance);
				fromnewdistance = new Distance(newtaxa.id, curtaxa.id, beforetaxa.distance + before_distance);
			}
			else if (aftertaxa.distance != BIGNUM)
			{
				newdistance = new Distance(curtaxa.id, newtaxa.id, aftertaxa.distance + after_distance);
				fromnewdistance = new Distance(newtaxa.id, curtaxa.id, aftertaxa.distance + after_distance);
			}
			else
			{
				newdistance = new Distance(curtaxa.id, newtaxa.id, BIGNUM);
				fromnewdistance = new Distance(newtaxa.id, curtaxa.id, BIGNUM);
			}

			curtaxa.compdist.addElement(newdistance);
			newtaxa.compdist.addElement(fromnewdistance);
			dummy = new Distance(curtaxa.id, newtaxa.id, BIGNUM);
			curtaxa.metricdist.addElement(dummy);
			dummy = new Distance(newtaxa.id, curtaxa.id, BIGNUM);
			newtaxa.metricdist.addElement(dummy);

			// We shouldnt need to add one to realdist because it only
			//  deals with real taxa
		}

		printclusters();
	}


	/*
	    jake may15-01 not my cod
	    Build Matris function places the values of the distances between nodes in the components vector.
	    Specifically it initiates the components.mindist= min distance between component.taxa[0] and all
	    other taxa, components.compdist a vector of distance obj between component.taxa[0] and all other
	    taxa, component.taxa[0].realdist/compdist/metricdist vectors of distance obj between .taxa[0] and
	    all other taxa.
	  */
	/**
	 *  Description of the Method
	 *
	 * @param  gapmode  Description of the Parameter
	 */
	private void BuildMatrix(boolean gapmode)
	{

		Component tempc1, tempc2;		// The two components we are going to compare
		Enumeration enum1   = components.elements();
		TaxaItem tempt1, tempt2;		// The two taxa we are going to compare
		Distance mindist    = new Distance();
		TaxaItem firsttaxa  = (TaxaItem) ((Component) components.elementAt(0)).taxa.elementAt(0);

		//this is important for .... entries
		while (enum1.hasMoreElements())
		{
			//first while
			tempc1 = (Component) enum1.nextElement();
			tempt1 = (TaxaItem) tempc1.taxa.elementAt(0);
			tempc1.mindist.distance = BIGNUM;

			Enumeration enum2  = components.elements();
			mindist.distance = BIGNUM;			// mindist will hold the minimum distance from c1 node to any other node

			while (enum2.hasMoreElements())
			{
				//second while
				tempc2 = (Component) enum2.nextElement();
				tempt2 = (TaxaItem) tempc2.taxa.elementAt(0);
				Distance realdist    = new Distance(tempc1.id, tempc2.id, 0); // tmpdist has the distance between these two

				// If it is myself, add the distance elements and go to top
				if (tempc1.id == tempc2.id)
				{
					Distance tmpcdist    = new Distance(tempc1.id, tempc2.id, 0, tempc1, tempc1);
					Distance tmptdist    = new Distance(tempt1.id, tempt2.id, 0, tempc1, tempc1);
					Distance metricdist  = new Distance(tempt1.id, tempt2.id, 0, tempc1, tempc1);
					tmpcdist.dc = tempc1;
					tmpcdist.sc = tempc1;
					tmptdist.dc = tempc1;
					tmptdist.sc = tempc1;
					realdist.dc = tempc1;
					realdist.sc = tempc1;
					tempc1.compdist.addElement(tmpcdist);
					tempt1.compdist.addElement(tmptdist);
					tempt1.realdist.addElement(realdist);
					tempt1.metricdist.addElement(metricdist);

					continue;
				}

//				Log.dprintln(32);
//				Log.dprintln("comparing " + tempt1.id + "," + tempt2.id, 32);

				for (int k = 0; k < tempt1.characters.length; k++)
				{

					char char1  = tempt1.characters[k];

					if (char1 == '.')
					{
						char1 = firsttaxa.characters[k];
					}

					char char2  = tempt2.characters[k];

					if (char2 == '.')
					{
						char2 = firsttaxa.characters[k];
					}

					// Log.dprint("[" + tempt1.characters[k] + " " + tempt2.characters[k] + "]");
					if (!gapmode && (char1 == '-' || char2 == '-'))
					{

						continue;
					}

					if (char1 == '?' || char2 == '?' || char1 == 'N' || char2 == 'N')
					{

						continue;
					}

					if (char1 != char2)
					{
						realdist.distance++;

						//MAPPING ADDITION NEEDED HERE
						//TABLE OF POSITIONS & CHARS
//						Log.dprint("increment to " + realdist.distance + " ", 0x20);
					}
				}

//				Log.dprintln(0x20);

				Distance tmpcdist    = new Distance(tempc1.id, tempc2.id, realdist.distance, tempc1, tempc2);
				Distance tmptdist    = new Distance(tempt1.id, tempt2.id, BIGNUM, tempc1, tempc2);
				Distance metricdist  = new Distance(tempt1.id, tempt2.id, BIGNUM, tempc1, tempc2);
				tempc1.compdist.addElement(tmpcdist);
				tempt1.compdist.addElement(tmptdist);
				tempt1.realdist.addElement(realdist);
				tempt1.metricdist.addElement(metricdist);

				if (realdist.distance < mindist.distance)
				{
					mindist.distance = realdist.distance; // mindist has taxa ids not component ids
					mindist.source = tempt1.id;
					mindist.sc = tempc1;
					mindist.destination = tempt2.id;
					mindist.dc = tempc2;
//					Log.dprintln("min distance for " + tempc1.id + " = " + mindist.distance + " from " + mindist.source + " to " + mindist.destination, 0x20);
				}
			} //end second while
//			Log.dprintln("final min distance for " + tempc1.id + " = " + mindist.distance + " from " + mindist.source + " to " + mindist.destination, 0x20);
			tempc1.mindist.clone(mindist);
			tempt1.minRealDist = mindist.distance;
		} //end first while
	}



	/**
	 *  Insert the method's description here. Creation date: (1/12/00 10:35:02
	 *  PM)
	 *
	 * @param  j   int
	 * @param  m   Description of the Parameter
	 * @param  it  Description of the Parameter
	 * @return     double
	 */

	/*
	    DP wrote it.
	    The CalcPars function is called by the buildParseTable function.  It integrates according
	    to some algorithm that I am not aware of and returns a result that enables the b.P.T to
	    determine the max distance.
	  */
	private double CalcPars(int j, int m, int it)
	{

		double i;
		double product;
		double dq;
		double pq;
		double num1;
		double num2;
		double q;
		double int1;
		double int2;
		double t1;
		double tr;
		double tp;
		int b           = 3;
		int r           = 1;
		double u        = 1.0;
//		Log.dprintln("j= " + j + " m= " + m + " it= " + it, 0x10000);
		product = 1.0;
		dq = u / it;

		/*
		    Product
		  */
		tr = (double) 1.0 / (b * r);
		tp = (double) 2.0 * m + 1.0;

		for (i = 0.00000001; i < j; i += 1.0)
		{

			/*
			    Integrate
			  */
			int1 = 0.0;
			int2 = 0.0;

			for (q = 0.0000001; q < u; q += dq)
			{
				t1 = (double) 1.0 - q * tr;
				num2 = Math.pow(q, i) * Math.pow(((double) 1.0 - q), tp) * t1 * Math.pow(t1 + 1.0 - q, i) * ((double) 1.0 - 2 * q * t1) * dq;
				num1 = num2 * q;
				int1 += num1;
				int2 += num2;
			}

			pq = (double) 1.0 - int1 / int2;
			product *= pq;
		}

		return (product);
	}


	/**
	 *  Connect the clusters to each other 3) Combine the two clusters and
	 *  recalculate the minimum distances to other clusters Creation date:
	 *  (10/26/99 2:50:04 PM)
	 *
	 * @param  tempc1  Description of the Parameter
	 * @param  tempc2  Description of the Parameter
	 */
	private void CombineClusters(Component tempc1, Component tempc2)
	{

		TaxaItem tempt1;
		Enumeration enum1;

		// Now join the two components into one.
//		Log.dprintln("---------- CombineClusters ", 0x40000);
		printcluster(tempc1);
		printcluster(tempc2);

		if (tempc1 == tempc2)
		{

			return;
		}

		enum1 = tempc2.taxa.elements();

		while (enum1.hasMoreElements())
		{

			// take the taxa out of the old cluster and put it in the new one
			tempt1 = (TaxaItem) enum1.nextElement();
			tempc1.taxa.addElement(tempt1);
			tempt1.parent = tempc1;
		}

		int removeId       = tempc2.id;

		// remove the second component
		components.removeElement(tempc2);

		// Recalculate the min connections
		// For clusters not involved in this collapse,
		// they need only point to the remaining cluster if the one
		// they were pointing to was removed.
		// We need to go through all of the taxa in the collapsed cluster to find out what the min connection is
		recalcMinDistance(tempc1, removeId);
	}


	/**
	 *  Connect the clusters to each other 1) find the two clusters that are
	 *  closest 2) insert intermediate nodes in such a way as to preserve the
	 *  maximum number of distances between taxa 3) Combine the two clusters and
	 *  recalculate the minimum distances to other clusters Creation date:
	 *  (10/26/99 2:50:04 PM)
	 *
	 * @return    Description of the Return Value
	 */
	private int ConnectClusters()
	{

		TaxaItem tempt1;
		TaxaItem tempt2;
		Component tempc1;
		Component tempc2;
		Distance mindist       = new Distance();
		Enumeration enum1;
		Enumeration enum2;

		// This vector will hold all of the destination taxa and the intermediates that have allready been added
		Vector destCandidates  = new Vector();

		// Find the two components that have the minimum distance
		// Look for minimum distance between clusters
		mindist.distance = BIGNUM;
		enum1 = components.elements();

		while (enum1.hasMoreElements())
		{
			tempc1 = (Component) enum1.nextElement();
//			Log.dprintln("finding min cluster comparing against " + tempc1.mindist.distance + " , current " + mindist.distance, 0x400);

			if (tempc1.mindist.distance < mindist.distance)
			{
//				Log.dprintln("updating min cluster to " + tempc1.mindist.distance + " , current was " + mindist.distance, 0x400);
				mindist.clone(tempc1.mindist);
			}
		}

		if (mindist.distance > maxdistance)
		{
//			Log.dprintln("Min distance " + mindist.distance + " is greater than maxdistance " + maxdistance + " not connecting", 0x40000);

			return (0);
		}

		// Connect every pair of taxa that are at this min distance.
		// Now connect the two real taxa from these two components that have the min distance
		// Delete the second component and copy the other taxa into the first component
		tempc1 = (Component) mindist.sc;
		tempc2 = (Component) mindist.dc;
//		Log.dprintln("---------- ConnectClusters selected dist" + mindist.distance, 0x40000);
		printcluster(mindist.sc);
		printcluster(mindist.dc);

		// First initialize the candidate vector with all of the valid destination taxa
		// Additional intermediates will be added to this vector each time we call
		//  bestMetric so we need a place to store them between invocations
		enum2 = mindist.dc.taxa.elements();

		while (enum2.hasMoreElements())
		{
			tempt2 = (TaxaItem) enum2.nextElement();
			destCandidates.addElement(tempt2);
		}

		enum1 = tempc1.taxa.elements();

		while (enum1.hasMoreElements())
		{
			tempt1 = (TaxaItem) enum1.nextElement();

			// If this is a real taxa, not an intermediate
			if (!tempt1.isIntermediate)
			{
				enum2 = tempc2.taxa.elements();

				while (enum2.hasMoreElements())
				{
					tempt2 = (TaxaItem) enum2.nextElement();

					// If this is a real taxa, not an intermediate
					if (!tempt2.isIntermediate)
					{

						// If this distance is the min distance, add the connection to the list
						Distance thisdist  = (Distance) tempt1.realdist.elementAt(tempt2.id);

						if (thisdist.distance == mindist.distance)
						{
							Log.dprintln("adding two min taxa " + thisdist.source + "[" + tempt1.name + "] , " + thisdist.destination + "[" + tempt2.name + "]", 0x8000);
							Log.dprintln("In clusters " + tempc1.id + " , " + tempc2.id + " distance " + mindist.distance, 0x8000);

							//if((tempt1.id == 12) && (tempt2.id == 0)) {
							//debug = 0x3604;
							//printclusters();
							//}
							Distance curdist  = (Distance) tempt1.compdist.elementAt(tempt2.id);

							if (curdist.distance <= mindist.distance)
							{
//								Log.dprintln("Connection already exists of length " + curdist.distance + " when trying to connect length " + mindist.distance, 0x40000);

								continue;
							}

							bestclust++;
//							Log.dprintln("connectClusters = " + bestclust, 0x10000);
							bestMetric(tempc1, tempc2, tempt1, tempt2, mindist.distance, destCandidates);
						}

						//end if equal dist
					}

					//end if not intermediate tempt2
				}

				//end while enum2
			}

			//end if not intermediate tempt1
		}

		//end while enum1
		// Now join the two components into one.
		CombineClusters(tempc1, tempc2);

		return (1);
	}


	/*
	    jake I did not write this code may18-01
	    ConnectComponents apears to add the intermediates necessary when two cluster combine.  It takes what
	    the evaluate metric decided was the best connection pts and actually connects them.
	  */
	/**
	 *  Description of the Method
	 *
	 * @param  sourcec     Description of the Parameter
	 * @param  destc       Description of the Parameter
	 * @param  newdist     Description of the Parameter
	 * @param  candidates  Description of the Parameter
	 */
	private void ConnectComponents(Component sourcec, Component destc, Distance newdist, Vector candidates)
	{

		TaxaItem sourcet;
		TaxaItem destt;
		TaxaItem nextt;
		TaxaItem newt;
//		Log.dprintln("---------- ConnectComponents dist " + newdist.distance, 0x40000);
		printcluster(sourcec);
		printcluster(destc);
		sourcet = (TaxaItem) alltaxa.elementAt(newdist.source);
		destt = (TaxaItem) alltaxa.elementAt(newdist.destination);

		// Now insert intermediate nodes and make the connections
//		Log.dprintln("adding connection" + newdist.source + "[" + sourcet.name + "] , " + newdist.destination + "[" + destt.name + "] distance " + newdist.distance, 0x40000);
		nextt = sourcet;

		int index;
		int intermediates;

		if (newdist.distance == 1)
		{

			// No intermediates needed
			sourcet.nbor.addElement(destt);
			destt.nbor.addElement(sourcet);
		}
		else
		{
			intermediates = newdist.distance;

			while (intermediates > 1)
			{

				// Insert intermediates
				intermediates--;

				// Add intermediate
				//jake ch 02/07/01
				newt = new TaxaItem("In" + Inum, 0, 0);

				// Insert him into the alltaxa vector
				Inum++;

				//////////////jake add 02/07/01
				alltaxa.addElement(newt);
				candidates.addElement(newt);

				// Insert him into the source cluster
				sourcec.taxa.addElement(newt);

				// find out who the new element is
				index = alltaxa.indexOf(newt);
				newt.id = index;
				newt.isIntermediate = true;

				// Make him my neighbor
				nextt.nbor.addElement(newt);
				newt.nbor.addElement(nextt);

				// add him to all of the distance vectors (in call to AddIntermediate
				// We only have non-infinite distances if the taxa used to have
				//  a connection to nextt or destt,
				//   if connected to nextt it is the distance to nextt+1
				//   if connected to destt it is the distance to destt+dist.distance
				// Otherwise, it is infinite
				AddIntermediate(newt, nextt, 1, destt, intermediates);

				// Move on to the next intermediate
				nextt = newt;

				if (intermediates == 1)
				{

					// This is the last intermediate in the chain
					// Now connect the last intermediate in the chain to the destination
					destt.nbor.addElement(newt);
					newt.nbor.addElement(destt);
				}
			}

			//end while intermediates >1
		}

		//end else
		// Now update the compdist fields in the source and destination to reflect the changes
		// Reflect these so that both clusters see them.
		// Sourcet is the current one from the source we are working with,
		// Destt is the current taxa in the destination component where the
		// connection was made.
		// This should probably be optimized later.
		// Iterate over all taxa in the source cluster
		Enumeration senumdist  = sourcec.taxa.elements();

		while (senumdist.hasMoreElements())
		{

			TaxaItem sourcetdist   = (TaxaItem) senumdist.nextElement();
			Distance oldsdist      = (Distance) sourcetdist.compdist.elementAt(newdist.source);
//			Log.dprintln("recalc source=" + sourcetdist.id, 0x8);

			// If there was some kind of a connection to the border taxa before
			// Update the distance between this node and all others in the opposite cluster
			Enumeration denumdist  = destc.taxa.elements();

			// Iterate over all taxa in the destination cluster
			while (denumdist.hasMoreElements())
			{

				TaxaItem desttdist  = (TaxaItem) denumdist.nextElement();
				int moddist;
//				Log.dprintln("recalc dest=" + desttdist.id, 0x8);

				Distance oldddist   = (Distance) desttdist.compdist.elementAt(newdist.destination);

				// If there was some kind of a connection to the border taxa before
				// Update the distance between this node and all others in the opposite cluster
				if ((oldsdist.distance == BIGNUM) || (oldddist.distance == BIGNUM))
				{

					continue;
				}
				else
				{

					Distance changedist  = (Distance) sourcetdist.compdist.elementAt(desttdist.id);
					moddist = oldddist.distance + oldsdist.distance + newdist.distance;

					if (moddist < changedist.distance)
					{
//						Log.dprintln("changing distances between " + changedist.source + " and " + changedist.destination + " from " + changedist.distance + " to " + moddist, 0x8);
						changedist.distance = moddist;
					}

					changedist = (Distance) desttdist.compdist.elementAt(sourcetdist.id);

					if (moddist < changedist.distance)
					{
//						Log.dprintln("changing distances between " + changedist.source + " and " + changedist.destination + " from " + changedist.distance + " to " + moddist, 0x8);
						changedist.distance = moddist;
					}
				}

				//end else
			}

			//end while denumdist
		}

		//end while senumdist
	}


	/**
	 *  Connect the source taxa to all other taxa with the minimum distance 1)
	 *  find the other taxa that are at a minimum distance 2) If a connection
	 *  doesnt already exist at the min distance, make it Creation date:
	 *  (10/26/99 2:50:04 PM) Changed 2/16/01
	 *
	 * @param  source  Description of the Parameter
	 */
	private void ConnectTaxa(TaxaItem source)
	{

		TaxaItem clustertaxa;
		Component tempc1;
		Component tempc2;
		Component cluster;
		Distance mindist             = new Distance();
		Distance dist;
		Enumeration enum1;
		Enumeration enumclusters;
		Enumeration enumclustertaxa;
		int combineclusters;

		/*
		    if (source.id == 156)
		    {
		    debug = 0x48600;
		    }
		    else
		    debug = 0x0;
		  */
//		Log.dprintln("::::::::::::::::ConnectTaxa: id" + source.id, 0x40000);

		// Look for minimum distance between this taxa and all others
		mindist.distance = BIGNUM;
		enum1 = source.realdist.elements();

		while (enum1.hasMoreElements())
		{
			dist = (Distance) enum1.nextElement();
//			Log.dprintln("finding min other taxa comparing against " + dist.distance + " , current " + mindist.distance, 0x200);

			if ((dist.distance < mindist.distance) && (dist.destination != source.id))
			{
//				Log.dprintln("updating min cluster to " + dist.distance + " , current was " + mindist.distance, 0x200);
				mindist.clone(dist);
			}
		}

		//if the mindist.distance is > maxdistance than it is not connected to any node
		if (mindist.distance > maxdistance)
		{
//			Log.dprintln("Min distance " + mindist.distance + " is greater than maxdistance " + maxdistance + " not connecting", 0x40000);

			return;
		}

		// Now add connections to all of the taxa at this min distance
		// Add them one cluster at a time, combining the clusters when you have finished with all the taxa connections within
		// that cluster
		// You only have to connect the source taxa to the other taxa, you dont have to make all possible min distance
		//   connections between clusters
		enumclusters = components.elements();

		// Look at all clusters
		while (enumclusters.hasMoreElements())
		{
			cluster = (Component) enumclusters.nextElement();

			// We have to look at other taxa in our cluster as well.
			// We out to think about this though since we normally assume that the connection is between two taxa
			//        if(cluster == source.parent)
			//          continue;
//			Log.dprintln("::::::::::::::::ConnectTaxa: source id %d " + source.id + " dest cluster id " + cluster.id, 0x40000);

			// Look at all of the taxa in the cluster
			combineclusters = 0;
			enumclustertaxa = cluster.taxa.elements();

			// Look at all the taxa in this cluster
			while (enumclustertaxa.hasMoreElements())
			{
				clustertaxa = (TaxaItem) enumclustertaxa.nextElement();

				if (clustertaxa.isIntermediate)
				{

					continue;
				}

				// Now look at distances from the source taxa to all of the taxa in the cluster
				dist = (Distance) source.realdist.elementAt(clustertaxa.id);

				// Find the distance to the taxa in this cluster
//				Log.dprintln("::::::::::::::::ConnectTaxa: source id " + source.id + " dest id " + cluster.id + " other taxa " + clustertaxa.id + "distance" + dist.distance, 0x40000);

				if (mindist.distance == dist.distance)
				{

					// Add this connection
					combineclusters = 1;

					TaxaItem sourcet           = source;
					TaxaItem destt             = (TaxaItem) alltaxa.elementAt(dist.destination);
//					Log.dprintln("------- ConnectTaxa Add min connection from " + source.id + "[" + source.name + "] to " + dist.destination + "[" + destt.name + "] distance " + dist.distance, 0x40000);

					// If there is already a connection, do nothing
					Distance already           = (Distance) source.compdist.elementAt(dist.destination);

					if (already.distance <= mindist.distance)
					{
//						Log.dprintln("CT We already have a connection between " + source.id + " and " + destt.id + " len " + already.distance + " not adding " + dist.distance, 0x200);
						printclusters();

						continue;
					}

//					Log.dprintln("two min taxa " + dist.source + "[" + sourcet.name + "] , " + dist.destination + "[" + destt.name + "]", 0x40000);

					// Connect these two in all of the ways that have the best metric
					// Add the intermediates before looking at another connection
					tempc1 = (Component) sourcet.parent;
					tempc2 = (Component) destt.parent;

					int limit                  = sourcet.minRealDist;

					if (limit < destt.minRealDist)
					{
						limit = destt.minRealDist;
					}

					// This vector will hold all of the destination taxa and the intermediates that have allready been added
					Vector destCandidates      = new Vector();
					Enumeration candidateEnum;
					TaxaItem candidateTaxa;

					// Initialize the candidate vector with all of the valid destination taxa
					// We dont really need to do this since we only make one connection between these two taxa
					//  but bestMetric requires it.
					candidateEnum = tempc2.taxa.elements();

					while (candidateEnum.hasMoreElements())
					{
						candidateTaxa = (TaxaItem) candidateEnum.nextElement();
						destCandidates.addElement(candidateTaxa);
					}

//					Log.dprintln("two min dist " + sourcet.minRealDist + "," + destt.minRealDist + "limit " + limit, 0x40000);
					besttaxa++;
//					Log.dprintln("connectTaxa " + besttaxa, 0x10000);
					bestMetric(tempc1, tempc2, sourcet, destt, limit, destCandidates);
				}

				// End of distances matched
			}

			// End of all of the taxa in this cluster
			// If we found at least one taxa at the minimum distance, combine them
			if (combineclusters == 1)
			{
//				Log.dprintln("+++++++++---- ConnectTaxa Calling CombineClusters ", 0x40000);
				printcluster(source.parent);
				printcluster(cluster);
				CombineClusters(source.parent, cluster);
				combineclusters = 0;
			}

			///////////////note problem combineclusters is never set off of 1====================================
		}

		// End of looking through all of the clusters
	}


	/**
	 *  Evaluate the metric based on the number of correct distances that can be
	 *  preserved. We start with the shortest connection between the two clusters
	 *  and move on up, incrementing the metric for every one that works.
	 *  Creation date: (11/4/99 12:11:04 AM)
	 *
	 * @param  sourcec           Description of the Parameter
	 * @param  destc             Description of the Parameter
	 * @param  numIntermediates  Description of the Parameter
	 * @param  limit             Description of the Parameter
	 * @return                   Description of the Return Value
	 */
	private int EvaluateMetric(Component sourcec, Component destc, int numIntermediates, int limit)
	{

		// Check to see how many work
		// This needs to be optimized, for now we will just search for the minimal
		//  real distance and increment a counter each time one works
		Enumeration enumsourcet;
		Enumeration enumdestt;
		TaxaItem sourceitem;
		TaxaItem destitem;
		Distance realdist;
		Distance metricdist;
		Distance compdist;
		int counter                       = 0;

		// The reasoning here is that you really only care that the closest
		// taxa is correct.  We weight the others on a sliding basis.
		final int CORRECT_SCORE           = 20;

		// Add this if the scores match
		final int TOO_BIG_SCORE           = 10;

		// just add this nomatter how much too big it is
		final int NUM_INTERMEDIATE_SCORE  = 1;

		// Add this amount for each intermediate
		// We want this small enough that it will only differentiate between insertions
		//  that are the same in other ways, but one has more intermediates
		// We have to look at all of the connections within clusters as well as between
		//Vector svect = new Vector();
		//Vector dvect = new Vector();
		Vector allvect                    = new Vector();

		// Make sure than none of the connections between clusters are
		// less than the limit, also make sure that the connection doesnt
		// make any taxa closer than their minimum
		int taxacnt1                      = 0;
		int taxacnt2                      = 0;
//		Log.dprintln("Evaluate Metric for [" + sourcec.id + "," + destc.id + "]", 0x10000);
		enumsourcet = sourcec.taxa.elements();

		while (enumsourcet.hasMoreElements())
		{
			sourceitem = (TaxaItem) enumsourcet.nextElement();

			//svect.addElement(sourceitem);
			allvect.addElement(sourceitem);

			//        dvect.addElement(sourceitem);
			taxacnt1++;
		}

		if (sourcec.id != destc.id)
		{
			enumdestt = destc.taxa.elements();

			while (enumdestt.hasMoreElements())
			{
				destitem = (TaxaItem) enumdestt.nextElement();

				//            svect.addElement(destitem);
				//dvect.addElement(destitem);
				allvect.addElement(destitem);
				taxacnt2++;
			}
		}

		/*
		    jake may 17-01
		  */
		if (taxacnt1 > taxacnt2)
		{
			taxacnt1 = taxacnt1 * 10;
		}
		else
		{
			taxacnt1 = taxacnt2 * 10;
		}

		final int TOO_SMALL_SCORE         = taxacnt1;

		// multiply this by the difference
		// Go through all of the taxa in the source and dest components
		// Look at all connections that have been made to look for violations
		//, comparing the metricdist with the real dist and count up the number
		// of connections that worked
		counter = -(numIntermediates - 1) * NUM_INTERMEDIATE_SCORE;

		//    enumsourcet = svect.elements();
		enumsourcet = allvect.elements();

		while (enumsourcet.hasMoreElements())
		{
			sourceitem = (TaxaItem) enumsourcet.nextElement();

			if (sourceitem.isIntermediate)
			{

				continue;
			}

			//        enumdestt = dvect.elements();
			enumdestt = allvect.elements();

			while (enumdestt.hasMoreElements())
			{
				destitem = (TaxaItem) enumdestt.nextElement();

				if (destitem.isIntermediate)
				{

					continue;
				}

				if (sourceitem.id == destitem.id)
				{

					continue;
				}

				realdist = (Distance) sourceitem.realdist.elementAt(destitem.id);
				metricdist = (Distance) sourceitem.metricdist.elementAt(destitem.id);
				compdist = (Distance) sourceitem.compdist.elementAt(destitem.id);

				// Dont bother with connections that are too far away
				if ((realdist.distance > (2 * maxdistance)) && (metricdist.distance > (2 * maxdistance)))
				{

					continue;
				}

				//here is where we are having problems,  we are not getting non negative distances for all connections between clusters -jake 05/10/01
//				Log.dprintln(" mindistcheck[" + realdist.source + "," + realdist.destination + "] metric " + metricdist.distance + " real " + realdist.distance + " counter " + counter + " limit " + limit + " minsrc " + sourceitem.minRealDist + " mindest " + destitem.minRealDist, 0x400);

				if (realdist.distance == metricdist.distance)
				{
					counter += CORRECT_SCORE;
//					Log.dprintln(" correct score between " + realdist.source + "," + realdist.destination, 0x400);
				}
				else
				{

					// Dont allow errors that are closer than our mindist
//					Log.dprintln(" smallcheck[" + realdist.source + "," + realdist.destination + "] metric " + metricdist.distance + " real " + realdist.distance + " counter " + counter + " limit " + limit, 0x400);

					if (realdist.distance > metricdist.distance)
					{

						if ((sourceitem.parent != destitem.parent) && (metricdist.distance <= limit))
						{
//							Log.dprintln(" deny1[" + realdist.source + "," + realdist.destination + "] metric " + metricdist.distance + " real " + realdist.distance + " counter " + counter + " limit " + limit, 0x400);
							counter = -BIGNUM;

							return counter;
						}
						else
						{

							double exp_factor;

							// We want the penalty to be really large for short connections and fairly small for big connections
							// The problem is that we still want a connection without "too small" connections to be better, even
							//  if it has a lot fewer correct connections.  For now just treat all errors the same.
							// exp_factor = (double)(realdist.distance - metricdist.distance)/(double)realdist.distance;
							exp_factor = (double) (realdist.distance - metricdist.distance);
							counter -= TOO_SMALL_SCORE * exp_factor;
//							Log.dprintln(" else too small [" + realdist.source + "," + realdist.destination + "] metric " + metricdist.distance + " real " + realdist.distance + " comp " + compdist.distance + " counter " + counter + " limit " + limit, 0x400);
						}

						// Make sure that connections within one of the clusters didnt get worse
						// I think that this should only be a penalty if it gets smaller than the minimum distance.  We wont check it here
						//                    if((compdist.distance < BIGNUM) &&
						//                          (compdist.distance > metricdist.distance)) {
						// ALREADY COMMMENTED OUT!
						//Log.dprintln(" deny2["+realdist.source+","+realdist.destination+"] metric "+metricdist.distance+" real "+realdist.distance+" comp "+compdist.distance+" counter "+counter+" limit "+limit,0x400);
						//                        counter -= BAD_GOT_SMALLER;
						//                       }
					}

					//end if dist>dist
					else
					{

						// Too big
//						Log.dprintln(" too big[" + realdist.source + "," + realdist.destination + "] metric " + metricdist.distance + " real " + realdist.distance + " comp " + compdist.distance + " counter " + counter + " limit " + limit, 0x400);
						counter -= TOO_BIG_SCORE;

						//metricdist.distance - realdist.distance;
					}
				}

				//end else
				// Now check to make sure that no minimum distances have been violated
				if (metricdist.distance < sourceitem.minRealDist)
				{
//					Log.dprintln(" deny3[" + realdist.source + "," + realdist.destination + "] metric " + metricdist.distance + " real " + realdist.distance + " counter " + counter + " minRealDist " + sourceitem.minRealDist, 0x400);
					counter = -BIGNUM;

					return counter;
					// Dont allow errors that are closer than our mindist
				}

//				Log.dprintln("end [" + realdist.source + "," + realdist.destination + "] metric " + metricdist.distance + " real " + realdist.distance + " counter " + counter + " limit " + limit, 0x400);
			}

			//end while enumdestt
		}

		//end while enumsourcet
		return (counter);
	}


	/**  Description of the Method */
	// DP wrote it
	private void OutWeights()
	{

		// calculates outgroup weights for each haplotype per each network
		Enumeration enumc      = components.elements();
		int numNetworks        = components.size() + 1;
		Enumeration enumn;
		Enumeration enumt;
		Component tempc;
		TaxaItem tempt;
		TaxaItem tempn;
		double[] total_weight  = new double[numNetworks];
		int networkNum         = 0;
		int ntaxa              = 0;
		Log.dprint("\n\nOUTGROUP WEIGHTS");

		while (enumc.hasMoreElements())
		{
			tempc = (Component) enumc.nextElement();
			enumt = tempc.taxa.elements();
			networkNum++;

			while (enumt.hasMoreElements())
			{
				tempt = (TaxaItem) enumt.nextElement();

				// Check out this node's neighbors
				if (!tempt.isIntermediate)
				{
					ntaxa++;
					enumn = tempt.nbor.elements();

					int sum_nbor_dup  = 0;
					int nnbors        = 0;
					int anynbor       = 0;

					//            Log.dprintln("\n\ncurrent taxa: " + tempt.name);
					while (enumn.hasMoreElements())
					{
						tempn = (TaxaItem) enumn.nextElement();

						if (!tempn.isIntermediate)
						{
							nnbors++;
							sum_nbor_dup += (tempn.numduplicates + 1);

							//    Log.dprintln("neighbor: " + tempn.name + " sum_nbor_dup: " + sum_nbor_dup);
						}

						anynbor++;
					}

					//Log.dprintln("nnbors = " + nnbors);
					if (anynbor == 1)
					{
						tempt.isTip = true;
					}
					else
					{
						tempt.isTip = false;
					}

					if (tempt.isTip)
					{
						tempt.weight = (tempt.numduplicates + 1) / 2.0;
					}
					else
					{
						tempt.weight = tempt.numduplicates + 1 + sum_nbor_dup;
					}

					//Log.dprintln("abs weight = " + tempt.weight);
					total_weight[networkNum] += tempt.weight;
				}
			}
		}

		Enumeration enumcomp   = components.elements();

		// should be able to get the count of this enum
		maxWtaxa = new String[numNetworks];

		Enumeration enumtaxa;
		Component tempcomp;
		TaxaItem temptaxa;
		double[] maxW          = new double[numNetworks];
		networkNum = 0;
		maxW[networkNum] = 0.0;
		outweights = new double[ntaxa];
		xlabels = new String[ntaxa];

		// can change this to not call this over and over
		while (enumcomp.hasMoreElements())
		{
			tempcomp = (Component) enumcomp.nextElement();
			enumtaxa = tempcomp.taxa.elements();
			networkNum++;

			int i  = 0;
			Log.dprintln("\n*** Network " + networkNum);
			maxW[networkNum] = 0.0;

			while (enumtaxa.hasMoreElements())
			{
				temptaxa = (TaxaItem) enumtaxa.nextElement();

				if (!temptaxa.isIntermediate)
				{
					temptaxa.oweight = temptaxa.weight / total_weight[networkNum];
					Log.dprintln(temptaxa.name + " weigth = " + temptaxa.oweight);
					outweights[i] = temptaxa.oweight;
					xlabels[i] = temptaxa.name;
					i++;

					if (temptaxa.oweight > maxW[networkNum])
					{
						maxW[networkNum] = temptaxa.oweight;
						maxWtaxa[networkNum] = temptaxa.name;
					}
				}
			}

			Log.dprintln("Total weight = " + total_weight[networkNum]);
			Log.dprintln("Biggest outgroup probability is " + maxWtaxa[networkNum] + " (" + maxW[networkNum] + ")");
		}
	}


	/**
	 *  Description of the Method
	 *
	 * @param  filename  Description of the Parameter
	 */
	private void PrintGraph(String filename)
	{
		// String outString      = "";  // added SW 7-15-03 to output in a standard format the network
		// outString += enumc.size();
		// outString += components.size();
		//

		Enumeration enumc     = components.elements();
		int numNetworks       = components.size() + 1;
		if (numNetworks > 2)
		{
			System.out.println("There are multiple networks (" + components.size() + ")!");

		}

		TaxaItem[] roots      = new TaxaItem[numNetworks];
		Enumeration enumt;

		// Enumeration enumn;
		Enumeration enumd;

		// Enumeration enum1;
		Component tempc;
		TaxaItem tempt;

		// TaxaItem tempn;
		// TaxaItem tempj;
		int namesize;
		int networkNum;
		TextOutputStream out  = new TextOutputStream(filename);

		// int num2              = 0;
		// int in                = 0;
		// int done              = 0;
		out.println("graph [");
		out.println("directed 0");
		networkNum = 0;

		// Print out all the nodes and their neighbors
		while (enumc.hasMoreElements())
		{
			tempc = (Component) enumc.nextElement();
			int numberOfNodes     = tempc.taxa.size();
			// outString += numberOfNodes + "\n";
			enumt = tempc.taxa.elements();
			networkNum++;

			while (enumt.hasMoreElements())
			{
				tempt = (TaxaItem) enumt.nextElement();

				if (distances)
				{
					tempt.numduplicates = 0;
				}

				//
				// Print out the node
				Log.dprint(tempt.name + "[" + tempt.id + "]: ");
				out.println("node [");
				out.println("   id " + tempt.id);
				// outString += id + " " + id + "\n";

				if (tempt.isIntermediate)
				{

					if (onOffinter == 0)
					{
						out.println("   label \" \"");
						out.println("   graphics [");
						out.println("   width 6.0");  //these give the old format
						out.println("   height 6.0");
					}
					else
					{
						out.println("   label \"" + tempt.name + "\"");
						out.println("   graphics [");
						out.println("   width 28.0");
						out.println("   height 18.0");
					}

					/////////////////////////////////////////////////jake 10-27-00
					out.println("   depth 1.0");
					out.println("]");
					out.println("   data [");
					out.println("   ]");
					out.println("   vgj [");
					out.println("      labelPosition \"center\"");
					out.println("   ]");
				}
				else
				{
					out.println("   label \"" + tempt.name + "\"");
					out.println("   vgj [");

					if (tempt.name.equalsIgnoreCase(maxWtaxa[networkNum]))
					{
						roots[networkNum] = tempt;

						// added for finding direction
						out.println("   shape  \"rectangle\"");
					}

					out.println("      labelPosition \"center\"");
					out.println("   ]");
					out.println("   graphics [");
					namesize = 9 * maxNameLen + tempt.numduplicates * 5;
					out.println("   width " + namesize);
					namesize = 20 + tempt.numduplicates * 5;
					out.println("   height " + namesize);
					out.println("   depth 20.0");
					out.println("]");
					out.println("   data [");
					out.println("    Frequency \"frequency=" + (tempt.numduplicates + 1));
					out.println(" " + tempt.name);

					// Print out this taxa duplicates
					enumd = tempt.dupnames.elements();

					while (enumd.hasMoreElements())
					{

						String tempd  = (String) enumd.nextElement();
						out.println(" " + tempd);
					}

					out.println("\"");
					out.printf("    Weight \"outgroup weight= %6.2f", tempt.oweight);
					out.println("\"");
					out.println("   ]");
					out.println("   vgj [");
					out.println("      labelPosition \"center\"");
					out.println("   ]");
				}

				out.println("]");

				// Print out this nodes neighbors
				// enumn = tempt.nbor.elements();
				// while (enumn.hasMoreElements())
				// {
				// tempn = (TaxaItem) enumn.nextElement();
				// System.out.println("edge [");
				// System.out.println("   source " + tempt.id);
				// System.out.println("   target " + tempn.id);
				// System.out.println("]");
				// }
			}

			// for this each element of this specific connected network
			// this particular connected part (Component)
			boolean hasAmbiguity  = directAndPrintEdges(out, roots[networkNum]);
			Log.dprintln("Ambiguous = " + hasAmbiguity);
			/*
			    System.out.println("Ambiguous = " + hasAmbiguity);
			  */
		}

		// end of for each connected element of the overall graph
		out.println("]");
		out.close();
	}


	/**
	 *  Gets the haplotypes attribute of the dna object
	 *
	 * @param  hapnum  Description of the Parameter
	 * @return         The haplotypes value
	 */
	public String getHaplotypes(int hapnum)
	{
		// return components.elementAt(index).;

		// Enumeration enumt;
		Enumeration enumd;
		Component tempc;
		TaxaItem tempt;
		String hapList     = "";
		// Log.dprintln("\nHaplotype list:\n\n");

		// while (enumc.hasMoreElements())
		// {
		// tempc = (Component) components.elementAt(0);
		// return tempc.taxa;
		// enumt = tempc.taxa.elementAt(index);

		tempt = getTaxa(new Integer(hapnum));
		// tempt = (TaxaItem) enumt.nextElement();
		// hapList += tempt.name + " : ");
		enumd = tempt.dupnames.elements();

		hapList += (tempt.dupnames.size() + 1) + "\n" + tempt.name;

		while (enumd.hasMoreElements())
		{
			String tempd  = (String) enumd.nextElement();
			hapList += "\n" + tempd;
		}
		return hapList;
	}

	//this prints out the names of taxa from the component list and all dups names
	/**  Description of the Method [DP] */
	private void PrintHaplotypeList()
	{

		Enumeration enumc  = components.elements();
		Enumeration enumt;
		Enumeration enumd;
		Component tempc;
		TaxaItem tempt;
		Log.dprintln("\nHaplotype list:\n\n");

		while (enumc.hasMoreElements())
		{
			tempc = (Component) enumc.nextElement();
			enumt = tempc.taxa.elements();

			while (enumt.hasMoreElements())
			{
				tempt = (TaxaItem) enumt.nextElement();
				Log.dprint(" - " + tempt.name + " : ");
				enumd = tempt.dupnames.elements();

				while (enumd.hasMoreElements())
				{
					String tempd  = (String) enumd.nextElement();
					Log.dprint(" " + tempd);
				}

				Log.dprintln();
			}
		}

		Log.dprintln();
	}


	/**
	 *  Description of the Method
	 *
	 * @param  matrixfile  Description of the Parameter
	 */
	private void PrintMatrix(String matrixfile)
	{

		//thecase = 0 realdist, 1 real and calcdist
		/////////print matrix completely rewriten 02/07/01 jake
		/*
		    int badDebug       = 0;
		    if (debug == 0)
		    {
		    debug = 0x1;
		    badDebug = 1;
		    }
		  */
		TaxaItem tempt;
		TaxaItem tempt2;

		//Enumeration enum1 = components.elements();
		Enumeration enum1;

		// = alltaxa.elements();
		Enumeration enum2;
		Distance dist;
		Distance dist2;
		int tabs           = 0;

		//    if (matrixfile != null)
		//        out = new TextOutputStream(matrixfile);
		//    else
		//        out = System.out;

		Log.dprintln("\n\nREAL");
		Log.dprintln("DISTANCE MATRIX");

		//Log.dprintln("components # of elements " + num);
		//Log.dprint("\nthis is a real (observed among sequences) distance matrix \n");
		Log.dprint("pos:   ");
		enum1 = realtaxa.elements();
		tempt = (TaxaItem) enum1.nextElement();
		tabs = tempt.name.length() / 2;

		for (int i = 0; i < tabs; i++)
		{
			Log.dprint("\t");
		}

		Log.dprint(" ");
		enum1 = realtaxa.elements();

		while (enum1.hasMoreElements())
		{

			//tempc1 = (Component) enum1.nextElement();
			tempt = (TaxaItem) enum1.nextElement();

			if ((tempt.id + 1) > 9)
			{
				Log.dprint(" " + (tempt.id + 1) + " ");
			}
			else
			{
				Log.dprint("  " + (tempt.id + 1) + " ");
			}
		}

		Log.dprint("\n");
		enum1 = realtaxa.elements();

		while (enum1.hasMoreElements())
		{

			//tempc1 = (Component) enum1.nextElement();
			//Enumeration enum2 = components.elements();
			//Enumeration
			enum2 = realtaxa.elements();

			//tempt = (TaxaItem)tempc1.taxa.elementAt(0) ;
			tempt = (TaxaItem) enum1.nextElement();
			Log.dprint("[" + tempt.name + "]");

			//Log.dprint(Integer.toString(tempcl.id+1));
			Log.dprint(Integer.toString(tempt.id + 1));

			if (tempt.id < 9)
			{
				Log.dprint(" ");
			}

			Log.dprint("\t:");

			while (enum2.hasMoreElements())
			{

				//tempc2 = (Component) enum2.nextElement();
				tempt2 = (TaxaItem) enum2.nextElement();

				//if (tempc1.id != tempc2.id) {
				if (tempt.id != tempt2.id)
				{

					//dist = (Distance) tempc1.compdist.elementAt(tempc2.id);
					dist = (Distance) tempt.realdist.elementAt(tempt2.id);

					if (dist.distance > 9)
					{
						Log.dprint(" " + dist.distance + " ");
					}
					else
					{
						Log.dprint("  " + dist.distance + " ");
					}
				}
				else
				{
					Log.dprint(" -- ");
				}
			}

			Log.dprint("\n");
		}



		Log.dprintln("\n\nCOMPUTED FROM THE NETWORK");
		Log.dprintln("DISTANCE MATRIX");

		//Log.dprintln("components # of elements " + num);
		//Log.dprint("\nthis is a computed (from the cladogram) patristic distance matrix \n");
		Log.dprint("pos:   ");
		enum1 = realtaxa.elements();
		tempt = (TaxaItem) enum1.nextElement();
		tabs = tempt.name.length() / 2;

		for (int i = 0; i < tabs; i++)
		{
			Log.dprint("\t");
		}

		Log.dprint(" ");
		enum1 = realtaxa.elements();

		while (enum1.hasMoreElements())
		{

			//tempc1 = (Component) enum1.nextElement();
			tempt = (TaxaItem) enum1.nextElement();

			if ((tempt.id + 1) > 9)
			{
				Log.dprint(" " + (tempt.id + 1) + " ");
			}
			else
			{
				Log.dprint("  " + (tempt.id + 1) + " ");
			}
		}

		Log.dprint("\n");
		enum1 = realtaxa.elements();

		while (enum1.hasMoreElements())
		{

			//tempc1 = (Component) enum1.nextElement();
			//Enumeration enum2 = components.elements();
			enum2 = realtaxa.elements();

			//tempt = (TaxaItem)tempc1.taxa.elementAt(0) ;
			tempt = (TaxaItem) enum1.nextElement();
			Log.dprint("[" + tempt.name + "]");

			//Log.dprint(Integer.toString(tempcl.id+1));
			Log.dprint(Integer.toString(tempt.id + 1));

			if (tempt.id < 9)
			{
				Log.dprint(" ");
			}

			Log.dprint("\t:");

			while (enum2.hasMoreElements())
			{

				//tempc2 = (Component) enum2.nextElement();
				tempt2 = (TaxaItem) enum2.nextElement();

				//if (tempc1.id != tempc2.id) {
				if (tempt.id != tempt2.id)
				{

					dist = (Distance) tempt.compdist.elementAt(tempt2.id);

					if (dist.distance == 1000000000)
					{
						Log.dprint(" " + "##" + " ");
					}
					else if (dist.distance > 9)
					{
						Log.dprint(" " + dist.distance + " ");
					}
					else
					{
						Log.dprint("  " + dist.distance + " ");
					}
				}
				else
				{
					Log.dprint(" -- ");
				}
			}

			Log.dprint("\n");
		}



		int matrixdiff     = 0;
		int matrixminus    = 0;
		Log.dprintln("\n\nDIFFERENCE MATRIX");

		//Log.dprintln("components # of elements " + num);
		//Log.dprint("\nthis is a REAL distance matrix - the COMPUTED distance matrix\n");
		Log.dprint("pos:   ");
		enum1 = realtaxa.elements();
		tempt = (TaxaItem) enum1.nextElement();
		tabs = tempt.name.length() / 2;

		for (int i = 0; i < tabs; i++)
		{
			Log.dprint("\t");
		}

		Log.dprint(" ");
		enum1 = realtaxa.elements();

		while (enum1.hasMoreElements())
		{

			//tempc1 = (Component) enum1.nextElement();
			tempt = (TaxaItem) enum1.nextElement();

			if ((tempt.id + 1) > 9)
			{
				Log.dprint(" " + (tempt.id + 1) + " ");
			}
			else
			{
				Log.dprint("  " + (tempt.id + 1) + " ");
			}
		}

		Log.dprint("\n");
		enum1 = realtaxa.elements();

		while (enum1.hasMoreElements())
		{

			//tempc1 = (Component) enum1.nextElement();
			//Enumeration enum2 = components.elements();
			//Enumeration
			enum2 = realtaxa.elements();

			//tempt = (TaxaItem)tempc1.taxa.elementAt(0) ;
			tempt = (TaxaItem) enum1.nextElement();
			Log.dprint("[" + tempt.name + "]");

			//Log.dprint(Integer.toString(tempcl.id+1));
			Log.dprint(Integer.toString(tempt.id + 1));

			if (tempt.id < 9)
			{
				Log.dprint(" ");
			}

			Log.dprint("\t:");

			while (enum2.hasMoreElements())
			{

				//tempc2 = (Component) enum2.nextElement();
				tempt2 = (TaxaItem) enum2.nextElement();

				//if (tempc1.id != tempc2.id) {
				if (tempt.id != tempt2.id)
				{

					dist = (Distance) tempt.realdist.elementAt(tempt2.id);
					dist2 = (Distance) tempt.compdist.elementAt(tempt2.id);

					if (dist2.distance == 1000000000)
					{
						Log.dprint(" ## ");
					}
					else if ((dist2.distance - dist.distance) > 9)
					{
						Log.dprint(" " + (dist2.distance - dist.distance) + " ");

						if (dist2.distance - dist.distance < 0)
						{
							matrixminus += dist2.distance - dist.distance;
						}
						else
						{
							matrixdiff += dist2.distance - dist.distance;
						}
					}
					else if ((dist2.distance - dist.distance) < 0)
					{
						Log.dprint(" " + (dist2.distance - dist.distance) + " ");

						if (dist2.distance - dist.distance < 0)
						{
							matrixminus += dist2.distance - dist.distance;
						}
						else
						{
							matrixdiff += dist2.distance - dist.distance;
						}
					}
					else
					{
						Log.dprint("  " + (dist2.distance - dist.distance) + " ");

						if (dist2.distance - dist.distance < 0)
						{
							matrixminus += dist2.distance - dist.distance;
						}
						else
						{
							matrixdiff += dist2.distance - dist.distance;
						}
					}
				}
				else
				{
					Log.dprint(" -- ");
				}
			}

			Log.dprint("\n");
		}

		Log.dprintln("\nThe total positive difference matrix is " + (matrixdiff / 2));
		Log.dprintln("\nThe total negative difference matrix is " + (matrixminus / 2));

		/*
		    if (badDebug == 1)
		    {
		    debug = 0x0;
		    }
		  */
	}



	/**
	 *  Recalculate the metric distances between clusters sourcec and destc when
	 *  a connection of "length" is made between taxa "sourcet" and "destt" We
	 *  dont actually add the intermediates. Just put the specified length in the
	 *  distance field for the metric and recalculate distances between clusters.
	 *  Creation date: (11/3/99 11:33:00 PM)
	 *
	 * @param  sourcec        Component
	 * @param  destc          Component
	 * @param  sourcebordert  Description of the Parameter
	 * @param  destbordert    Description of the Parameter
	 * @param  length         int
	 */
	private void RecalcDist(Component sourcec, Component destc, TaxaItem sourcebordert, TaxaItem destbordert, int length)
	{

		// sourcec is the source component
		// destc is the destination component
		// sourcebordert is the taxa on the border that we are going to connect to a taxa in the dest cluster
		// destbordert is the taxa on the border that we are going to connect to a taxa in the source cluster
		// First set the distance between sourcet and destt
		// length is the distance between the two border taxa
		Distance sd              = (Distance) sourcebordert.metricdist.elementAt(destbordert.id);
		sd.distance = length;

		// We need to update distances within the clusters that might have changed as well
		// So we create two vectors with all of the taxa from both clusters in them.
		Vector svect             = new Vector();
		Vector dvect             = new Vector();
		Enumeration enumsourcet;
		Enumeration enumdestt;
		TaxaItem sourceitem;
		TaxaItem destitem;
		enumsourcet = sourcec.taxa.elements();

		while (enumsourcet.hasMoreElements())
		{
			sourceitem = (TaxaItem) enumsourcet.nextElement();
			svect.addElement(sourceitem);
			dvect.addElement(sourceitem);
		}

		if (sourcec.id != destc.id)
		{
			enumdestt = destc.taxa.elements();

			while (enumdestt.hasMoreElements())
			{
				destitem = (TaxaItem) enumdestt.nextElement();
				svect.addElement(destitem);
				dvect.addElement(destitem);
			}
		}

		// Now update all of the distances
		Distance dist;
		Distance compdist;
		int newdistance;
		enumsourcet = svect.elements();

		while (enumsourcet.hasMoreElements())
		{
			sourceitem = (TaxaItem) enumsourcet.nextElement();
			enumdestt = dvect.elements();

			while (enumdestt.hasMoreElements())
			{
				destitem = (TaxaItem) enumdestt.nextElement();
//				Log.dprintln("RCD sourcec " + sourcec.id + " destc " + destc.id + " sourceitem " + sourceitem.id + " destitem " + destitem.id + "sourceborder" + sourcebordert.id + " destborder " + destbordert.id + " length " + length, 16);

				// set distance = dist(here->border) + dist(border->border-on-other-cluster) +
				//  dist(border-on-other-cluster->other taxa)
				dist = (Distance) sourceitem.metricdist.elementAt(destitem.id);
				compdist = (Distance) sourceitem.compdist.elementAt(destitem.id);
				newdistance = ((Distance) sourceitem.compdist.elementAt(sourcebordert.id)).distance;

				// source to border
//				Log.dprintln("distance from " + dist.source + "[" + sourceitem.name + "] to " + sourcebordert.id + "[" + sourcebordert.name + "] = " + newdistance, 16);
				newdistance += length;

				//source border to dest border
//				Log.dprintln("distance from " + dist.source + " to " + destbordert.id + "[" + destbordert.name + "] = " + newdistance, 16);
				newdistance += ((Distance) destbordert.compdist.elementAt(destitem.id)).distance;

				// border to dest
//				Log.dprintln("distance between " + sourceitem.id + " and " + destitem.id + "[" + destitem.name + "] set to " + newdistance, 16);
//				Log.dprintln("updating distance between " + sourceitem.id + " and " + destitem.id + " set to " + newdistance, 16);

				// If this change didnt make things closer, leave the old distance in the metric
				if (newdistance < compdist.distance)
				{
					dist.distance = newdistance;
				}
				else
				{
					dist.distance = compdist.distance;
				}

//				Log.dprintln("final distance between " + sourceitem.id + " and " + destitem.id + " set to " + dist.distance, 16);
			}
		}
	}


	/**
	 *  Unmark all of the metric distances so they will not be finalized later
	 *  Creation date: (11/3/99 11:06:44 PM)
	 *
	 * @param  source  Component
	 * @param  dest    Component
	 */
	private void RemoveAllMetrics(Component source, Component dest)
	{

		Enumeration enumt;
		TaxaItem item;

		// First do the source cluster
		enumt = source.taxa.elements();

		while (enumt.hasMoreElements())
		{
			item = (TaxaItem) enumt.nextElement();
			item.newconnections.removeAllElements();
		}

		// Now do the destination cluster
		enumt = dest.taxa.elements();

		while (enumt.hasMoreElements())
		{
			item = (TaxaItem) enumt.nextElement();
			item.newconnections.removeAllElements();
		}
	}


	/**
	 *  jake may17-01 I did not write this code bestMetric is designed to try and
	 *  produce the best connections between two taxa source and dest it tries to
	 *  use the nodes source and dest intermediates to connect. first calling
	 *  recalcdist, next evaluateMetric, and finally connectComponents if there
	 *  is a new better connection found Creation date: (12/15/99 8:35:28 AM)
	 *
	 * @param  sourcec         Description of the Parameter
	 * @param  destc           Description of the Parameter
	 * @param  sourcet         Description of the Parameter
	 * @param  destt           Description of the Parameter
	 * @param  limit           Description of the Parameter
	 * @param  destCandidates  Description of the Parameter
	 */
	private void bestMetric(Component sourcec, Component destc, TaxaItem sourcet, TaxaItem destt, int limit, Vector destCandidates)
	{

		Enumeration enums  = sourcec.taxa.elements();
		TaxaItem tempt1;
		TaxaItem tempt2;
		Enumeration enum2;
		Distance bestDist  = new Distance(0, 0, 0);
		int metric         = -BIGNUM;
		int firstTime      = 1;
		long now1;
		long now0;
		long now2;
		long now3;
		int evalcount      = 0;
		now0 = System.currentTimeMillis();

		// Look through everything in the source cluster
		while (enums.hasMoreElements())
		{

			// If this is the origin taxa, or another intermediate in the cluster, it is a candidate
			tempt1 = (TaxaItem) enums.nextElement();

			if ((tempt1 == sourcet) || tempt1.isIntermediate)
			{

				// To everything in the dest cluster
				enum2 = destCandidates.elements();

				while (enum2.hasMoreElements())
				{

					// If this is the destination taxa, or another intermediate in the cluster, it is a candidate
					tempt2 = (TaxaItem) enum2.nextElement();

					if ((tempt2 == destt) || tempt2.isIntermediate)
					{

						// Connect the source and destination through these two nodes
						// If the source and destination are in different clusters, we can
						//  treat these existing intermediates as border nodes.  If the two are
						//  in the same cluster, we have to take the distance to the closest one and
						//  treat the closest existing intermediate as the border.
						int total         = ((Distance) sourcet.realdist.elementAt(destt.id)).distance;
//						Log.dprintln("trying to connect " + sourcet.id + "[" + sourcet.name + "] to " + destt.id + "[" + destt.name + "] through " + tempt1.id + "[" + tempt1.name + "] and " + tempt2.id + "[" + tempt2.name + "]" + "limit=" + limit, 0x200);

						// Evaluate the connection of the components using "total" number of intermediates
						// Recalculate distances, the only ones that will change are the
						//  distances where one item was in tempc1 and the other was in tempc2
						//  The distance for each item is the distance from that item to tempt1
						//   + the distance from tempt1 to tempt2 + the distance from tempt2 to
						//   the item on the other side.  You never need to do a full shortest
						//   path calculation
						// When we are in the same cluster, we can not only add new intermediates,
						//  but we can create a shorter path by directly connecting
						printclusters();

						Distance nearest  = (Distance) sourcet.compdist.elementAt(tempt1.id);

						//nearest holds the distance from the source node to some boundary intermediate node
						Distance other    = (Distance) destt.compdist.elementAt(tempt2.id);

						//other holds the distance from the destination node to some boundary node in it's cluster
						// We dont need to check for distances to the other intermediates (sourcet-tempt2)
						//  since we will check it later.
						// If the distance between the endpoints and the intermediates is already too big,
						//  we dont have a chance
						total -= nearest.distance + other.distance;

						//if total is less than one that the intermediates that we have choosen are to far apart
						if (total < 1)
						{

							continue;
							//this goes back to top of while
						}

//						Log.dprintln("looking at connection " + tempt1.id + " and " + tempt2.id + " distance " + total, 0x200);
						RecalcDist(sourcec, destc, tempt1, tempt2, total);
//						Log.dprintln("after recalcdist " + tempt1.id + " and " + tempt2.id + " distance " + total, 0x200);

						// Evaluate the metric.  You only need to look at pairs distributed
						//  across the two clusters
						now1 = System.currentTimeMillis();
//						Log.dprintln("trying to connect " + sourcet.id + "[" + sourcet.name + "] to " + destt.id + "[" + destt.name + "] through " + tempt1.id + "[" + tempt1.name + "] and " + tempt2.id + "[" + tempt2.name + "]" + "# intermediates " + limit, 0x10000);
						evalcount++;

						int res           = EvaluateMetric(sourcec, destc, total, limit);
						now2 = System.currentTimeMillis();
//						Log.dprintln("It took " + (now1 - now2) / 60000.0 + " minutes.", 0x10000);
//						Log.dprintln("metric was " + res + " min was " + metric, 0x200);

						if ((firstTime == 1) || (res > metric))
						{

							// Unmark previous connections since they are worse than this one
							firstTime = 0;
//							Log.dprintln("trying to connect " + sourcet.id + "[" + sourcet.name + "] to " + destt.id + "[" + destt.name + "] through " + tempt1.id + "[" + tempt1.name + "] and " + tempt2.id + "[" + tempt2.name + "]" + "limit=" + limit, 0x200);
							bestDist.source = tempt1.id;
							bestDist.destination = tempt2.id;
							bestDist.distance = total;
							metric = res;
//							Log.dprintln("updating best between " + tempt1.id + " and " + tempt2.id + ", metric =" + metric + " total " + total, 0x200);
						}
						else
						{
//							Log.dprintln("Not Keeping this connection min=" + metric + "this configuration=" + res, 0x200);
						}
					}

					//end if tempt2==destt || is intermediate
				}

				//end while enum2
			}

			//end if tempt1==sourcet || is intermediate
		}

		//end while enums
		tempt1 = (TaxaItem) alltaxa.elementAt(bestDist.source);
		tempt2 = (TaxaItem) alltaxa.elementAt(bestDist.destination);
//		Log.dprintln("End of BestMetric!!!!!!!!!!!!!!!!!!", 0x10000);
		now3 = System.currentTimeMillis();
//		Log.dprintln("It took " + (now3 - now0) / 60000.0 + " minutes.", 0x10000);
		bestime += (double) (now3 - now0);
//		Log.dprintln("bestime = " + bestime, 0x10000);
//		Log.dprintln("Number of evaluateMetric's = " + evalcount + "\n\n", 0x10000);

		Distance already   = (Distance) tempt1.compdist.elementAt(tempt2.id);

		if (already.distance <= bestDist.distance)
		{
//			Log.dprintln("not adding intermediates between " + tempt1.id + " and " + tempt2.id + ", metric =" + metric + " was already " + already.distance + " wanted to make it " + bestDist.distance, 0x200);
		}
		else
		{
//			Log.dprintln("adding intermediates between " + tempt1.id + " and " + tempt2.id + ", metric =" + metric + " was already " + already.distance + " wanted to make it " + bestDist.distance, 0x200);
			ConnectComponents(sourcec, destc, bestDist, destCandidates);
		}
	}


	/**
	 *  Insert the method's description here. Creation date: (1/12/00 10:03:29
	 *  PM)
	 *
	 * @param  MaxSequences    Description of the Parameter
	 * @param  MaxNucleotides  Description of the Parameter
	 * @param  conLimit        Description of the Parameter
	 * @return                 int
	 */

	/*
	    jake may 15-01 I didn't write this code. DP wrote it. SW modified it
	    It appears to be a loop that calls CalcPars to integrate the according to the number of
	    nucleotides in the sequence.  The goal is to set a limit on the maximum distance between
	    two nodes.  If the minimum distance between a node and any other node is greater than the
	    maxdistance found by buildParseTable() then it is an unconnected node.
	  */
	private int buildParseTable(int MaxSequences, int MaxNucleotides, int conLimit)
	{

		int index            = 0;
		double parsval       = 1;

		if (frame != null)
		{
			frame.statustextField.setText("STATUS: Calculating " + conLimit + "% connection limit");
			frame.statustextField.update(frame.statustextField.getGraphics());
		}

		Log.dprintln("_________________________________________________________");
		Log.dprintln("\n\nPARSIMONY PROBABILITY");

		double percentValue  = (conLimit * .01);

		/*
		    if (conLimit == 0)
		    {
		    //         percentValue = .0001;
		    Log.dprintln("Since 0% chosen, used 10000 for connection limit.");
		    return 10000;
		    }
		  */
		while (parsval > percentValue)
		{

			// this may go on forever with a connection limit of 0 or close to that...
			index++;
			parsval = CalcPars(index, MaxNucleotides - index, 2000);
			Log.dprintln("For " + index + " step(s),\tP(" + conLimit + "%) = " + parsval);

			//         System.out.println("For " + index + " step(s),\tP(" + conLimit + "%) = " + parsval);
		}

		Log.dprintln("");

		//      String a = String.valueOf(index - 1);
		//      frame.textField.setText("95% connection limit = " + a + " steps");
		//      frame.textField.update(frame.textField.getGraphics());
		//frame.manualConnections.setText(a);
		return (index - 1);
	}


	// public void dodna(String infile, String outfile, int args, boolean dist)
	// {
	/**
	 *  Description of the Method
	 *
	 * @param  infile       Description of the Parameter
	 * @param  outfile      Description of the Parameter
	 * @param  args         Description of the Parameter
	 * @param  dist         Description of the Parameter
	 * @param  logFileName  Description of the Parameter
	 */
	public void dodna(String infile, String outfile, int args, boolean dist, String logFileName)
	{

		String matrixfile     = "matrixfile.txt";
		Enumeration enumt;
		TaxaItem source;
		distances = dist;

		int progress          = 0;
		int val               = 0;
		float quantum;
		int currentIteration;
		long end1;
		bestclust = 0;
		besttaxa = 0;
		gapmode = true;
		//this is the first function that is called.  It starts the
		//process of creating a cladeogram from the .nex files.
		if ((args & 0x1) != 0)
		{
			gapmode = false;
		}

		bestime = 0.0;

		if (log)
		{
			Log = new Logger();

			// logfile = new TextOutputStream("logfile.txt");
			Log.setLogFile(logFileName);

		}

		boolean Gui           = false;

		if (frame != null)
		{
			Gui = true;
			frame.progressBar.setValue(0);
			frame.progressBar.update(frame.progressBar.getGraphics());
			start1 = System.currentTimeMillis();
			frame.statustextField.setText("STATUS: Reading file and collapsing");
			frame.statustextField.setForeground(Color.red);
			frame.statustextField.update(frame.statustextField.getGraphics());

			// frame = new JFrame();
		}

		Log.dprintln("TCS " + VERSION);

		Date time             = new Date();
		Log.dprintln(time.toString());
		Log.dprintln("Datafile = " + infile);

		Log.dprintln("Current OS = " + currentOS);
		
		// Read the input file
		fileReader = new FileReader(frame, Log);

		if (frame != null)
		{
			frame.progressBar.setValue(1);
			frame.progressBar.update(frame.progressBar.getGraphics());
		}

		if (distances)
		{
			fileReader.ReadDistanceFile(infile);
		}
		else
		{
			fileReader.ReadInputFile(infile, gapmode);

		}

		// if there is a gui or there wasn't a commandline maxDistance set
		if (Gui || !commandLineMax)
		{

			calculateConnectionLimit();
		}
		else
		{
			Log.dprintln("\nRUN SETTINGS");
			Log.dprintln("User specified maximum connection steps = " + maxdistance);
		}

		PrintTaxa();
		warnMissing = fileReader.warnMissing;
		missingString = fileReader.missingString;
		maxNameLen = fileReader.maxNameLen;
		components = fileReader.components;
		alltaxa = fileReader.alltaxa;
		realtaxa = fileReader.realtaxa;

		//this will put the char seq into components/alltaxa/realtaxa
		if (gapmode)
		{
			Log.dprintln("Gaps treated as fifth state");
		}
		else
		{
			Log.dprintln("Gaps treated as missing data");
		}

		if (warnMissing)
		{
			Log.dprintln("\n\nWARNING: POSSIBLE MISSING DATA AMBIGUITIES");
			Log.dprintln(missingString);
		}

		Log.dprintln("\n\nHAPLOTYPES");
		Log.dprintln("Number of haplotypes = " + components.size());

		if (components.size() == 1)
		{
			JOptionPane.showMessageDialog(frame, "There is only one haplotype" + "\nA graph cannot be built.", "TCS warning", JOptionPane.ERROR_MESSAGE);
			System.out.println("Only one Haplotype!");

			if (warnMissing)
			{
				JOptionPane.showMessageDialog(frame, "Warning : potential missing data ambiguities" + "\nwhen collapsing sequences to haplotypes" + "\nCheck the logfile.\n", "TCS warning", JOptionPane.ERROR_MESSAGE);
			}
			return;
			//         frame.dispose();
			// don't want to die if this happens I don't think...
//			System.exit(0);
		}

		if (!distances)
		{

			// Build the distance matrix
			PrintHaplotypeList();

			if (frame != null)
			{
				frame.statustextField.setText("STATUS: Calculating distances");
				frame.statustextField.update(frame.statustextField.getGraphics());
			}

			BuildMatrix(gapmode);
		}

		// Print the distance matrix
		//PrintMatrix(matrixfile);
		Log.dprintln("\n\nCONNECTIONS");
		quantum = (float) 50 / (float) realtaxa.size();
		currentIteration = 0;

		int middle            = 0;

		if (frame != null)
		{
			frame.statustextField.setText("STATUS: Estimating subnetworks");
			frame.statustextField.update(frame.statustextField.getGraphics());
			val = frame.progressBar.getValue();
		}

		// Now connect each taxa to the other taxa with minimum distance
		enumt = realtaxa.elements();

		while (enumt.hasMoreElements())
		{
			currentIteration++;

			if (frame != null)
			{
				progress = val + (int) (currentIteration * quantum);

				//        Log.dprintln("progress1 = "+progress);
				// Update progress bar
				frame.progressBar.setValue(progress);
				frame.progressBar.update(frame.progressBar.getGraphics());
			}

			source = (TaxaItem) enumt.nextElement();
//			Log.dprintln("connecting taxa " + source.id + "[" + source.name + "]", 0x40000);
			printclusters();

			//          if(source.id == 32)
			//            debug = 0xffeb;
			ConnectTaxa(source);

			if ((debug & 0x4000) != 0)
			{
				Log.dprintln("writing graph" + middle + ".out");
				PrintGraph("graph" + middle + ".out");
			}

			middle++;
		}

		quantum = (float) 50 / (float) realtaxa.size();
		currentIteration = 0;

		if (frame != null)
		{
			frame.statustextField.setText("STATUS: Connecting subnetworks");
			frame.statustextField.update(frame.statustextField.getGraphics());
		}

		// Now connect all remaining clusters
		while (components.size() > 1)
		{
			currentIteration++;

			if (frame != null)
			{
				progress = 50 + (int) (currentIteration * quantum);

				//        Log.dprintln("progress2 = "+progress);
				// Update progress bar
				frame.progressBar.setValue(progress);
				frame.progressBar.update(frame.progressBar.getGraphics());
			}

			printclusters();

			if ((debug & 0x4000) != 0)
			{
				Log.dprintln("writing graph" + middle + ".out");
				PrintGraph("graph" + middle + ".out");
			}

			middle++;

			if (ConnectClusters() == 0)
			{

				break;
			}
		}

		PrintMatrix(matrixfile);

		// Can we make at least 1 connection ? (i.e., is there a graph?). If not, do not continue
		graph_exists = false;

		Enumeration enumC     = components.elements();
		Enumeration enumT;
		Enumeration enumN;
		Component tempC;
		TaxaItem tempT;

		while (enumC.hasMoreElements())
		{
			tempC = (Component) enumC.nextElement();
			enumT = tempC.taxa.elements();

			while (enumT.hasMoreElements())
			{
				tempT = (TaxaItem) enumT.nextElement();
				enumN = tempT.nbor.elements();

				if (enumN.hasMoreElements())
				{
					graph_exists = true;
				}
			}
		}

		if (graph_exists)
		{

			// Now print out the graph
			OutWeights();
			Log.dprintln("\n\nGRAPH");
			Log.dprintln("Here is the graph");
			PrintGraph(outfile);
			Log.dprintln("\n\n\nCalculations are finished\n");
		}
		else
		{
			JOptionPane.showMessageDialog(frame, "No connection below limit." 
			+ "\nA graph cannot be built.", "TCS warning", JOptionPane.ERROR_MESSAGE);

			//         frame.dispose();
			System.exit(0);
		}

		end1 = System.currentTimeMillis();
		Log.dprintln("\n\n_________________________________________________________");
		Log.dprintln("TCS " + VERSION);

		Date time2            = new Date();
		Log.dprintln(time2.toString());
		Log.dprintln("Datafile = " + infile);
		Log.dprintln("\nIt took " + (end1 - start1) / 60000.0 + " minutes.");

		double temppercent    = 0;
//		Log.dprintln("the total time = " + (end1 - start1), 0x10000);
//		Log.dprintln("bestime = " + bestime, 0x10000);
//		Log.dprintln("# bestMetric calls for Taxa = " + besttaxa, 0x10000);
//		Log.dprintln("# bestMetric calls for Cluster = " + bestclust, 0x10000);
		temppercent = (bestime / (end1 - start1)) * 100;
//		Log.dprintln("\nPercent of time for best = " + temppercent + "%", 0x10000);

		if (frame != null)
		{
			frame.progressBar.setValue(100);
			frame.progressBar.update(frame.progressBar.getGraphics());
			// frame.weightsMenu.setEnabled(false);
			frame.statustextField.setText("STATUS: program finished");
			frame.statustextField.update(frame.statustextField.getGraphics());
		}

		Log.close();

		if (warnMissing)
		{
			JOptionPane.showMessageDialog(frame, "Warning : potential missing data ambiguities" 
			+ "\nwhen collapsing sequences to haplotypes" 
			+ "\nCheck the logfile.\n", "TCS warning", JOptionPane.ERROR_MESSAGE);
		}
	}


	/**  Description of the Method */
	private void calculateConnectionLimit()
	{

		String steps  = "";
		int conLimit  = 95;

		if (frame != null)
		{
			//
			if (frame.manualBox.getState())
			{
				steps = frame.manualConnections.getText();
			}
			else
			{
				// conLimit = frame.conLimit.getSelectedIndex();
				String conL  = frame.conLimit.getSelectedItem();
				conL = conL.substring(0, 2);

				conLimit = Integer.parseInt(conL);

			}

			// System.out.println("conLimit = " + conLimit);
			// need to do something other than this here
		}

		/*
		    Log.dprintln("\nSteps string = " + steps);
		  */
		//this if is typically taken jake may15-01
		if (steps.equals(""))
		{

			if (fileReader.doingHaps == true) // [DP]
			{
				maxdistance = fileReader.len;
				Log.dprintln("\rRUN SETTINGS");
				Log.dprintln("Maximum connection steps equal number of SNPs = " + maxdistance );
			}

			
			// Calculate ParsProb and print it
			maxdistance = buildParseTable(fileReader.num, fileReader.len, conLimit);
			Log.dprintln("\nRUN SETTINGS");
			Log.dprintln("Calculated maximum connection steps at " + conLimit + "% = " + maxdistance);

			if (frame != null)
			{
				frame.calcLimitLabel.setText("Connection Limit = " + maxdistance);
				frame.calcLimitLabel.update(frame.calcLimitLabel.getGraphics());
			}
		}
		else
		{
			try
			{
				maxdistance = Integer.parseInt(steps);
				if (maxdistance <= 0)
				{
					throw new Exception();
				}
				Log.dprintln("\nRUN SETTINGS");
				Log.dprintln("User specified maximum connection steps = " + maxdistance);
			}
			catch (Exception e)
			{

				JOptionPane.showMessageDialog(frame, "Error : You must use a positive integer for the Connection limit!\n\tUsing a default value of 40 for the Connection Limit", "TCS warning", JOptionPane.ERROR_MESSAGE);
				maxdistance = 40;
				Log.dprintln("\nRUN SETTINGS");
				Log.dprintln("Invalid user specified maximum connection steps.  Default = 40" + maxdistance);
				// return;
			}
			// Log.dprintln("\nRUN SETTINGS");
			// Log.dprintln("User specified maximum connection steps = " + maxdistance);
		}

		// Calculate ParsProb and print it
		//      maxdistance = buildParseTable(num, len);
		//      Log.dprintln("Maximum connection steps at 95% = " + maxdistance + "\n");
	}


	// gets all the information about the file used to create this graph...
	/**
	 *  Description of the Method
	 *
	 * @param  GMLo  Description of the Parameter
	 */
	public void fillTaxaInfo(GMLobject GMLo)
	{

		GMLobject nodegml;
		GMLobject gml;
		GMLobject GMLtmp;
		gml = GMLo;

		// If the GML doesn't contain a graph, assume it is a graph.
		GMLtmp = gml.getGMLSubObject("graph", GMLobject.GMLlist, false);

		if (GMLtmp != null)
		{
			gml = GMLtmp;
		}
		// go through and get all nodes...
		for (nodegml = gml.getGMLSubObject("node", GMLobject.GMLlist, false); nodegml != null; nodegml = gml.getNextGMLSubObject())
		{

			boolean intermediate  = false;
			String name           = null;
			int length            = 0;
			int id                = -1;
			Integer tmpint        = null;
			String tmp            = null;

			if ((tmp = (String) nodegml.getValue("label", GMLobject.GMLstring)) != null)
			{
				name = tmp;
			}

			if ((tmpint = (Integer) nodegml.getValue("id", GMLobject.GMLinteger)) != null)
			{
				id = tmpint.intValue();
			}

			GMLobject data;
			int freq              = 1;

			if ((data = nodegml.getGMLSubObject("data", GMLobject.GMLlist, false)) != null)
			{

				if ((tmp = (String) data.getValue("Frequency", GMLobject.GMLstring)) != null)
				{

					int first      = tmp.indexOf("=");
					int second     = tmp.indexOf("\n");
					String number  = tmp.substring(first + 1, second).trim();
					freq = Integer.parseInt(number);
				}
			}
			else
			{
				intermediate = true;
				freq = 1;
			}

			if (name.length() != 0)
			{

				TaxaItem taxa  = new TaxaItem(name, length, id);
				taxa.numduplicates = freq - 1;
				taxa.isIntermediate = intermediate;
				alltaxa.addElement(taxa);
			}
		}

		GMLobject edgegml;

		// now get all edges
		for (edgegml = gml.getGMLSubObject("edge", GMLobject.GMLlist, false); edgegml != null; edgegml = gml.getNextGMLSubObject())
		{

			Integer source;
			Integer target;
			source = (Integer) edgegml.getValue("source", GMLobject.GMLinteger);
			target = (Integer) edgegml.getValue("target", GMLobject.GMLinteger);

			if (source != null && target != null)
			{

				TaxaItem tSource  = getTaxa(source);
				TaxaItem tTarget  = getTaxa(target);
				tSource.nbor.add(tTarget);
				tTarget.nbor.add(tSource);
			}
		}
	}


	/**
	 *  Insert the method's description here. Creation date: (11/5/99 4:07:13 PM)
	 *
	 * @param  tempc  Description of the Parameter
	 */
	private void printcluster(Component tempc)
	{

		TaxaItem tempt;
		Enumeration enum;
		enum = tempc.taxa.elements();
//		Log.dprintln("cluster # " + tempc.id, 0x4);

		while (enum.hasMoreElements())
		{
			tempt = (TaxaItem) enum.nextElement();
//			Log.dprint("Taxa # " + tempt.id + "[" + tempt.name + "],", 0x4);
		}

//		Log.dprintln("", 0x4);
	}


	/**
	 *  Insert the method's description here. Creation date: (11/5/99 4:07:13 PM)
	 */
	private void printclusters()
	{
		if ((debug & 0x4) == 0)
		{

			return;
		}
		TaxaItem tempt1;
		TaxaItem tempt2;
		Component tempc1;
		Distance dist;
		Enumeration enum1;
		Enumeration enum2;
		Enumeration enum3;

		enum1 = components.elements();

		while (enum1.hasMoreElements())
		{
			tempc1 = (Component) enum1.nextElement();
			Log.dprintln();
			Log.dprintln("cluster # " + tempc1.id + " !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
			Log.dprintln("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
			enum2 = tempc1.taxa.elements();

			while (enum2.hasMoreElements())
			{
				tempt1 = (TaxaItem) enum2.nextElement();
				Log.dprintln("Taxa # " + tempt1.id + "[" + tempt1.name + "] compdist");
				enum3 = tempt1.compdist.elements();

				while (enum3.hasMoreElements())
				{
					dist = (Distance) enum3.nextElement();
					Log.dprint("([" + dist.source + "," + dist.destination + "]=" + dist.distance + ")");
				}

				Log.dprintln();
				Log.dprintln("Taxa # " + tempt1.id + " realdist");
				enum3 = tempt1.realdist.elements();

				while (enum3.hasMoreElements())
				{
					dist = (Distance) enum3.nextElement();
					Log.dprint("([" + dist.source + "," + dist.destination + "]=" + dist.distance + ")");
				}

				Log.dprintln();
				Log.dprintln("Taxa # " + tempt1.id + " metricdist");
				enum3 = tempt1.metricdist.elements();

				while (enum3.hasMoreElements())
				{
					dist = (Distance) enum3.nextElement();
					Log.dprint("([" + dist.source + "," + dist.destination + "]=" + dist.distance + ")");
				}

				Log.dprintln();
				Log.dprintln("Taxa # " + tempt1.id + "Neighbors");
				enum3 = tempt1.nbor.elements();

				while (enum3.hasMoreElements())
				{
					tempt2 = (TaxaItem) enum3.nextElement();
					Log.dprint("(" + tempt2.id + "[" + tempt2.name + "]),");
				}

				Log.dprintln();
			}

			if (tempc1.mindist.dc != null)
			{
				Log.dprintln("mindist= " + tempc1.mindist.distance + " between " + tempc1.mindist.source + " in " + tempc1.id + " and " + tempc1.mindist.destination + " in " + tempc1.mindist.dc.id);
			}
		}
	}


	/**
	 *  jake may 16 I didn't write recalcMinDistance updates the components
	 *  vectors mindist.dc. Those components whose mindist was pted to the
	 *  removed component are updated to pt to the collapsed component. The
	 *  collapsed component now looks for its new mindist to all other
	 *  components. It doesn't appear very efficient. I think that a comparison
	 *  between the mindist of the two components that were combined in combine
	 *  clusters would be more effective. Creation date: (11/6/99 10:59:44 AM)
	 *
	 * @param  collapse  Description of the Parameter
	 * @param  removeid  Description of the Parameter
	 */
	private void recalcMinDistance(Component collapse, int removeid)
	{

		Component tempc1;

		// The two components we are going to compare
		TaxaItem tempt1;

		// The two taxa we are going to compare
		TaxaItem tempt2;
		Distance mindist          = new Distance();
		Distance nextdist;
		Enumeration enumall;
		Enumeration enumtaxa;
		Enumeration enumcollapse;

		// first change all of the other clusters that used to have the removed cluster as their min,
		//  to now have the collapsed version as their min.
		enumall = components.elements();

		while (enumall.hasMoreElements())
		{
			tempc1 = (Component) enumall.nextElement();

			if (tempc1.mindist.dc.id == removeid)
			{
				tempc1.mindist.dc = collapse;
			}
		}

		mindist.distance = BIGNUM;

		// Now we need to find the min distance for this cluster.
		// Search through all of the other clusters for the taxa that has the min distance from here
		enumall = components.elements();

		// For each of the other clusters
		while (enumall.hasMoreElements())
		{
			tempc1 = (Component) enumall.nextElement();

			if (tempc1 == collapse)
			{

				continue;
			}

			enumtaxa = tempc1.taxa.elements();

			// For each of the real taxa in this cluster
			while (enumtaxa.hasMoreElements())
			{
				tempt1 = (TaxaItem) enumtaxa.nextElement();

				if (tempt1.isIntermediate)
				{

					continue;
				}

				enumcollapse = collapse.taxa.elements();

				// Compare with everything in collapse to find the minimum
				while (enumcollapse.hasMoreElements())
				{
					tempt2 = (TaxaItem) enumcollapse.nextElement();

					if (tempt2.isIntermediate)
					{

						continue;
					}

					nextdist = (Distance) tempt2.realdist.elementAt(tempt1.id);

					if (nextdist.distance < mindist.distance)
					{
						mindist.clone(nextdist);
						mindist.sc = collapse;
						mindist.dc = tempc1;
//						Log.dprintln("min distance for " + collapse.id + " = " + mindist.distance + " from " + mindist.source + " to " + mindist.destination + " in " + tempc1.id, 0x100);
					}

					//end if dist < mindist
				}

				//end while enumcollapse
			}

			//end while enum taxa
		}

		//end while enumall
//		Log.dprintln("recalc cluster min distance for " + collapse.id + " = " + mindist.distance + " from " + mindist.source + " to " + mindist.destination, 0x100);
		collapse.mindist = mindist;
	}


	/**
	 *  Gets the taxa attribute of the dna object
	 *
	 * @param  id  Description of the Parameter
	 * @return     The taxa value
	 */
	private TaxaItem getTaxa(Integer id)
	{

		int index  = id.intValue();
		int size   = alltaxa.size();

		for (int x = 0; x < size; x++)
		{

			TaxaItem tTmp  = (TaxaItem) alltaxa.elementAt(x);

			if (tTmp.id == index)
			{

				return tTmp;
			}
		}

		return null;
	}


	/*
	    Method to determine if all of a given internal node's neighbors have been visited in the Directing algorithm
	  */
	/**
	 *  Description of the Method
	 *
	 * @param  tempn  Description of the Parameter
	 * @return        Description of the Return Value
	 */
	private boolean HasUnvisitedNeighbors(TaxaItem tempn)
	{

		Vector neig  = tempn.nbor;
		int size     = neig.size();

		while (size > 0)
		{

			TaxaItem temp  = (TaxaItem) neig.elementAt(--size);

			if (!temp.visited)
			{

				return true;
			}
		}

		return false;
	}


	/**
	 *  Description of the Method
	 *
	 * @param  out   Description of the Parameter
	 * @param  root  Description of the Parameter
	 * @return       Description of the Return Value
	 */
	private boolean directAndPrintEdges(TextOutputStream out, TaxaItem root)
	{

		boolean isAmbig   = false;

		// int level         = 0;
		// get a minimum distance from root for each node, and a min dist to a leaf?
		//       use a stack to do a breadth first traversal (highest weight first?), going away from root if there is no abiguity...
		// root.levelNumber = level;
		root.levelNumber = 0;

		LinkedList queue  = new LinkedList();
		queue.addLast(root);

		// Number of steps away from the root
		while (queue.size() > 0)
		{

			TaxaItem cur       = (TaxaItem) queue.removeFirst();
			cur.visited = true;

			TaxaItem source;
			TaxaItem dest;
			Enumeration enumn  = cur.nbor.elements();
			TaxaItem tempn;
			int numNeighbors   = cur.nbor.size();
			int numEdges       = 0;

			// need to keep track of whether or not the node has been visited
			// enumn.hasMoreElements();
			// System.out.println("(" + cur.name + ") numNeighbors = " + numNeighbors);
			while (numEdges < numNeighbors)
			{

				//if enumn != null?
				tempn = (TaxaItem) enumn.nextElement();

				if (tempn.levelNumber == -1)
				{
					tempn.levelNumber = cur.levelNumber + 1;
				}

				if (!tempn.visited)
				{
					source = cur;
					dest = tempn;

					// System.out.println("numEdge visiting = " + numEdges + "(" + tempn.name + ")");
					if (tempn.levelNumber < cur.levelNumber)
					{

						// if cur is further from the root than it's child
						Log.dprintln("Weird!  Parent is farther from root than Child!");
					}

					if (tempn.levelNumber == cur.levelNumber)
					{

						// if they are at the same level (Also should mean that there is ambig!)
						/*
						    Log.dprintln("ambiguous I think!");
						  */
						// add to this all those where tempn is more than one level from cur
						if (tempn.isIntermediate != cur.isIntermediate)
						{

							/*
							    }
							    if (tempn.isIntermediate && !cur.isIntermediate)
							    {
							  */
							// make the intermed -> leaf
							if (tempn.isIntermediate)
							{
								/*
								    System.out.println("Intermediate and leaf connected at same level!");
								  */
								//this is not absolutely true... It seemed it had to be that way because otherwise I would get an internal nodes with no outgoing branch
								// at least with my algorithm...  that is probably the most likely...
								// However, if there is at least one outgoing, higher-level branch from the intermediate, it could be the other way...
								if (HasUnvisitedNeighbors(tempn))
								{

									// perhaps I should do a weight comparison?
									// System.out.println("The intermediate has unvisited neighbors... Will use weights to discriminate!");

									if (tempn.oweight > cur.oweight)
									{
										source = tempn;
										dest = cur;
									}
									else
									{

										if (tempn.oweight == cur.oweight)
										{
											// System.out.println("Two weights are identical!");
										}
									}
								}
								else
								{
									// System.out.println("Edge must go to root (cause all internal node's neighbors are visited!");
									source = tempn;
									dest = cur;
								}
							}
							else
							{

								// if cur is intermediate...
								// System.out.println("Intermediate and leaf connected at same level!");

								//this is not absolutely true... It seemed it had to be that way because otherwise I would get an internal nodes with no outgoing branch
								// at least with my algorithm...  that is probably the most likely...
								// However, if there is at least one outgoing, higher-level branch from the intermediate, it could be the other way...
								if (HasUnvisitedNeighbors(cur))
								{

									// perhaps I should do a weight comparison?
									// System.out.println("The intermediate has unvisited neighbors... Will use weights to discriminate!");

									if (tempn.oweight > cur.oweight)
									{
										source = tempn;
										dest = cur;
									}
									else
									{

										if (tempn.oweight == cur.oweight)
										{
											// System.out.println("Two weights are identical!");
										}
									}
								}
								else
								{
									// System.out.println("Edge must go to root (cause all internal node's neighbors are visited!");
									dest = tempn;
									source = cur;
								}
							}
						}
						else
						{

							/*
							    if (tempn.isIntermediate == cur.isIntermediate)
							  */
							// if both are intermediates or leaves
							// find the best weight
							//compare the weights (which one should be the root?)
							// rootest has greatest oweight
							if (cur.isIntermediate)
							{

								// both are intermediate...
								// System.out.println("Two at this level are intermediates!");
							}

							if (tempn.oweight > cur.oweight)
							{
								// System.out.println("Two at this level are leaves!");
								source = tempn;
								dest = cur;
							}
							else
							{

								if (tempn.oweight == cur.oweight)
								{
									// System.out.println("Two weights are identical!");
								}
							}
						}
					}

					out.println("edge [");
					out.println("   source " + source.id);
					out.println("   target " + dest.id);
					out.println("]");

					//sort the neighbors by the weight
					// then push them on the stack highest weight first
					// add them to the queue in the right order
					// only add if it is not a leaf, and it is not yet visited
					if (!queue.contains(tempn))
					{
						queue.addLast(tempn);
					}
					else
					{
						isAmbig = true;

						// System.out.println("" + tempn.name + " already in queue");
					}
				}
				else
				{

					// isAmbig = true;
				}

				numEdges++;
			}

			// System.out.println("" + numEdges + " edges for level " + level);
			// level++;
		}

		return isAmbig;
	}
}
